#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_fabric_span_session
version_added: "2.11.0"
short_description: Manage Fabric SPAN Sessions on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Switched Port Analyzer (SPAN) Sessions on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.4) and later.
author:
- Sabari Jaganathan (@sajagana)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Fabric Monitoring Access Policy template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the template.
    - The template must be a Fabric Monitoring Access Policy template.
    - This parameter or O(template) is required.
    type: str
  name:
    description:
    - The name of the SPAN Session.
    type: str
    aliases: [ span_session ]
  uuid:
    description:
    - The UUID of the SPAN Session.
    - This parameter is required when the O(name) needs to be updated.
    type: str
    aliases: [ span_session_uuid ]
  description:
    description:
    - The description of the SPAN Session.
    type: str
  admin_state:
    description:
    - The administrative state of the SPAN Session.
    - Defaults to C(enabled) when unset during creation.
    choices: [ enabled, disabled ]
    type: str
  mtu:
    description:
    - The MTU truncation size for the SPAN packets.
    - The value must be in the range 64 - 9216.
    - Defaults to 1518 when unset during creation.
    type: int
  destination_epg:
    description:
    - The destination EPG configuration group.
    - This parameter or O(destination_port) or O(destination_port_channel) is required when creating the SPAN Session.
    type: dict
    suboptions:
      epg_uuid:
        description:
        - The UUID of the destination EPG to use for the SPAN Session.
        - This parameter or O(destination_epg.epg) is required.
        type: str
      epg:
        description:
        - The destination EPG to use for the SPAN Session.
        - This parameter or O(destination_epg.epg_uuid) is required.
        type: dict
        suboptions:
          name:
            description:
            - The name of the destination EPG.
            type: str
            required: true
          template:
            description:
            - The name of the template that contains the destination EPG.
            - This parameter or O(destination_epg.epg.template_id) is required.
            type: str
          template_id:
            description:
            - The ID of the template that contains the destination EPG.
            - This parameter or O(destination_epg.epg.template) is required.
            type: str
          schema_id:
            description:
            - The ID of the schema that contains the destination EPG.
            - This parameter or O(destination_epg.epg.schema) is required.
            type: str
          schema:
            description:
            - The name of the schema that contains the destination EPG.
            - This parameter or O(destination_epg.epg.schema_id) is required.
            type: str
          anp:
            description:
            - The name of the ANP that contains the destination EPG.
            - This parameter or O(destination_epg.epg.anp_uuid) is required.
            type: str
          anp_uuid:
            description:
            - The UUID of the ANP that contains the destination EPG.
            - This parameter or O(destination_epg.epg.anp) is required.
            type: str
      destination_ip:
        description:
        - The destination IP address to route SPAN Session packets.
        type: str
      source_ip_prefix:
        description:
        - The prefix used to assign source IP addresses to ERSPAN packets which can be used to identify which Leaf or Spine is sending the traffic.
        - This can be any IP. If the prefix is used, the node ID of the source node is used for the undefined bits of the prefix.
        type: str
      span_version:
        description:
        - The version of the SPAN Session.
        - Defaults to C(v2) when unset during creation.
        choices: [ v1, v2 ]
        type: str
      enforce_span_version:
        description:
        - Enforce the SPAN Session version defined in O(destination_epg.span_version).
        - Defaults to true when unset during creation.
        type: bool
      flow_id:
        description:
        - The flow ID of the SPAN Session packets.
        - The value must be in the range 1 - 1023.
        - Defaults to 1 when unset during creation.
        type: int
      ttl:
        description:
        - The time to live (TTL) of the SPAN Session packets.
        - The value must be in the range 1 - 1023.
        - Defaults to 1 when unset during creation.
        type: int
      dscp:
        description:
        - The DSCP value for sending the monitored SPAN Session packets.
        - Defaults to C(unspecified) when unset during creation.
        choices:
          - af11
          - af12
          - af13
          - af21
          - af22
          - af23
          - af31
          - af32
          - af33
          - af41
          - af42
          - af43
          - cs0
          - cs1
          - cs2
          - cs3
          - cs4
          - cs5
          - cs6
          - cs7
          - expedited_forwarding
          - voice_admit
          - unspecified
        type: str
  destination_port:
    description:
    - The destination port configuration group.
    - This parameter or O(destination_epg) or O(destination_port_channel) is required when creating the SPAN Session.
    type: dict
    suboptions:
      port_uuid:
        description:
        - The UUID of the destination port to use for the SPAN Session.
        - This parameter or O(destination_port.port) is required.
        type: str
      port:
        description:
        - The destination port to use for the SPAN Session.
        - This parameter or O(destination_port.port_uuid) is required.
        type: dict
        suboptions:
          node:
            description:
            - The Node ID of the Node to use for the SPAN Session.
            type: int
            required: true
          interface:
            description:
            - The Ethernet interface of the Node to use for the SPAN Session
            type: str
            required: true
  destination_port_channel:
    description:
    - The destination port channel configuration group.
    - This parameter or O(destination_epg) or O(destination_port) is required when creating the SPAN Session.
    type: dict
    suboptions:
      port_channel_uuid:
        description:
        - The UUID of the destination port channel to use for the SPAN Session.
        - This parameter or O(destination_port_channel.port_channel) is required.
        type: str
      port_channel:
        description:
        - The destination port channel to use for the SPAN Session.
        - This parameter or O(destination_port_channel.port_channel_uuid) is required.
        type: dict
        suboptions:
          name:
            description:
            - The name of the destination port channel.
            type: str
            required: true
          template:
            description:
            - The name of the template that contains the destination port channel.
            - This parameter or O(destination_port_channel.port_channel.template_id) is required.
            type: str
          template_id:
            description:
            - The ID of the template that contains the destination port channel.
            - This parameter or O(destination_port_channel.port_channel.template) is required.
            type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Fabric Monitoring Access Policy template.
- The O(destination_epg.epg) must exist before using it with this module in your playbook.
  Use M(cisco.mso.mso_schema_template_anp_epg) to create the EPG.
- The O(destination_port_channel.port_channel) must exist before using it with this module in your playbook.
  Use M(cisco.mso.ndo_port_channel_interface) to create the Fabric resource port channel interface.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.mso_schema_template_anp_epg
- module: cisco.mso.ndo_port_channel_interface
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create Fabric SPAN Session with destination EPG
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_test_epg
    destination_epg:
      epg:
        schema: ansible_test
        template: Template1
        anp: Anp1
        name: EPG1
      destination_ip: "1.1.1.1"
      source_ip_prefix: "2.2.2.2"
    state: present
  register: create_epg_span_session

- name: Create Fabric SPAN Session with destination Port
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_test_port
    destination_port:
      port:
        node: 101
        interface: "eth1/1"
    state: present

- name: Create Fabric SPAN Session with destination Port Channel
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_test_pc
    destination_port_channel:
      port_channel:
        template: ansible_test
        name: ansible_test_resource_pc_1
    state: present

- name: Update Fabric SPAN Session from destination EPG to destination Port
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_test_epg
    destination_port:
      port:
        node: 101
        interface: "eth1/1"
    state: present

- name: Update Fabric SPAN Session from destination Port to destination Port Channel
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_test_port
    destination_port_channel:
      port_channel:
        template: ansible_test
        name: ansible_test_resource_pc_1
    state: present

- name: Update Fabric SPAN Session from destination Port Channel to destination EPG
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_test_pc
    destination_epg:
      epg:
        schema: ansible_test
        template: Template1
        anp: Anp1
        name: EPG1
      destination_ip: "1.1.1.1"
      source_ip_prefix: "2.2.2.2"
    state: present

- name: Update the Fabric SPAN Session name using UUID
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    uuid: "{{ create_epg_span_session.current.uuid }}"
    name: ansible_test_pc_updated
    state: present

- name: Query a specific Fabric SPAN Session using name
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_test_pc_updated
    state: query
  register: query_with_name

- name: Query a specific Fabric SPAN Session using UUID
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    uuid: "{{ create_epg_span_session.current.uuid }}"
    state: query
  register: query_with_uuid

- name: Query all Fabric SPAN Sessions
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    state: query
  register: query_all_objects

- name: Delete a specific Fabric SPAN Session using Name
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_test_pc_updated
    state: absent

- name: Delete a Fabric SPAN Session using UUID
  cisco.mso.ndo_fabric_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template_id: ansible_test
    uuid: "{{ create_epg_span_session.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, epg_object_reference_spec
from ansible_collections.cisco.mso.plugins.module_utils.schemas import MSOSchemas
from ansible_collections.cisco.mso.plugins.module_utils.templates import MSOTemplates
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.constants import TARGET_DSCP_MAP, ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        name=dict(type="str", aliases=["span_session"]),
        uuid=dict(type="str", aliases=["span_session_uuid"]),
        description=dict(type="str"),
        admin_state=dict(type="str", choices=["enabled", "disabled"]),
        mtu=dict(type="int"),
        destination_epg=dict(
            type="dict",
            mutually_exclusive=[
                ("epg", "epg_uuid"),
            ],
            required_one_of=[
                ["epg", "epg_uuid"],
            ],
            options=dict(
                epg_uuid=dict(type="str"),
                epg=epg_object_reference_spec(),
                destination_ip=dict(type="str"),
                source_ip_prefix=dict(type="str"),
                span_version=dict(type="str", choices=["v1", "v2"]),
                enforce_span_version=dict(type="bool"),
                flow_id=dict(type="int"),
                ttl=dict(type="int"),
                dscp=dict(type="str", choices=list(TARGET_DSCP_MAP)),
            ),
        ),
        destination_port=dict(
            type="dict",
            mutually_exclusive=[
                ("port", "port_uuid"),
            ],
            required_one_of=[
                ["port", "port_uuid"],
            ],
            options=dict(
                port_uuid=dict(type="str"),
                port=dict(
                    type="dict",
                    options=dict(
                        node=dict(type="int", required=True),
                        interface=dict(type="str", required=True),
                    ),
                ),
            ),
        ),
        destination_port_channel=dict(
            type="dict",
            mutually_exclusive=[
                ("port_channel", "port_channel_uuid"),
            ],
            required_one_of=[
                ["port_channel", "port_channel_uuid"],
            ],
            options=dict(
                port_channel_uuid=dict(type="str"),
                port_channel=dict(
                    type="dict",
                    options=dict(
                        name=dict(type="str", required=True),
                        template=dict(type="str"),
                        template_id=dict(type="str"),
                    ),
                    required_one_of=[
                        ["template", "template_id"],
                    ],
                    mutually_exclusive=[
                        ("template", "template_id"),
                    ],
                ),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[("template", "template_id"), ("destination_epg", "destination_port_channel", "destination_port")],
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
        required_one_of=[
            ["template", "template_id"],
        ],
    )

    mso = MSOModule(module)
    mso_schemas = MSOSchemas(mso)
    mso_templates = MSOTemplates(mso)

    template_name = module.params.get("template")
    template_id = module.params.get("template_id")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    admin_state = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(module.params.get("admin_state")) if module.params.get("admin_state") else None
    mtu = module.params.get("mtu")
    destination_epg = module.params.get("destination_epg")
    destination_port = module.params.get("destination_port")
    destination_port_channel = module.params.get("destination_port_channel")
    state = module.params.get("state")

    mso_template = MSOTemplate(mso, "monitoring_tenant", template_name, template_id)
    mso_template.validate_template("monitoring")
    object_description = "SPAN Session"
    site_id = mso_template.template.get("monitoringTemplate").get("sites")[0].get("siteId")

    match = None
    if uuid or name:
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            mso_template.template.get("monitoringTemplate", {}).get("template", {}).get("spanSessions", []),
            [KVPair("uuid", uuid) if uuid else KVPair("name", name)],
        )
    else:
        match = mso_template.template.get("monitoringTemplate", {}).get("template", {}).get("spanSessions", [])

    if (uuid or name) and match:
        set_fabric_span_session_object_details(mso_template, site_id, match.details)
        mso.existing = mso.previous = copy.deepcopy(match.details)  # Query a specific object
    elif match:
        mso.existing = [set_fabric_span_session_object_details(mso_template, site_id, obj) for obj in match]  # Query all objects

    if state != "query":
        span_session_path = "/monitoringTemplate/template/spanSessions/{0}".format(match.index if match else "-")

    ops = []

    if state == "present":
        if uuid and not mso.existing:
            mso.fail_json(msg="{0} with the UUID: '{1}' not found".format(object_description, uuid))

        mso_values = dict()
        if destination_epg:
            mso_values["destination"] = dict(
                remote=dict(
                    epgRef=mso_schemas.get_epg_uuid(destination_epg.get("epg"), destination_epg.get("epg_uuid")),
                    spanVersion=destination_epg.get("span_version"),
                    enforceSpanVersion=destination_epg.get("enforce_span_version"),
                    destIPAddress=destination_epg.get("destination_ip"),
                    srcIPPrefix=destination_epg.get("source_ip_prefix"),
                    flowID=destination_epg.get("flow_id"),
                    ttl=destination_epg.get("ttl"),
                    dscp=TARGET_DSCP_MAP.get(destination_epg.get("dscp")),
                ),
                mtu=mtu,
            )

        if destination_port:
            # Destination Port supports UUIDs, but we have no module to get them
            destination_port_uuid = destination_port.get("port_uuid")

            if destination_port_uuid is None:
                node = destination_port.get("port").get("node")
                interface_port = destination_port.get("port").get("interface")
                destination_port_uuid = mso.get_site_interface_details(site_id=site_id, uuid=None, node=node, port=interface_port).get("uuid")
            mso_values["destination"] = dict(local=dict(accessInterface=destination_port_uuid), mtu=mtu)

        if destination_port_channel:
            destination_port_channel_uuid = destination_port_channel.get("port_channel_uuid")
            if destination_port_channel_uuid is None:
                fabric_resource_template = mso_templates.get_template(
                    "fabric_resource",
                    destination_port_channel.get("port_channel").get("template"),
                    destination_port_channel.get("port_channel").get("template_id"),
                )

                destination_port_channel_uuid = fabric_resource_template.get_template_policy_uuid(
                    "fabric_resource", destination_port_channel.get("port_channel").get("name"), "portChannels"
                )

            mso_values["destination"] = dict(local=dict(portChannel=destination_port_channel_uuid), mtu=mtu)

        if match:
            mso_update_values = {"name": name, "description": description}
            mso_update_values[("destination", "mtu")] = mtu

            if admin_state is not None:
                mso_update_values[("sourceGroup", "enableAdminState")] = admin_state

            if destination_epg:
                if match.details.get("destination", {}).get("local", {}).get("accessInterface") or match.details.get("destination", {}).get("local", {}).get(
                    "portChannel"
                ):
                    mso_update_values[("destination", "local")] = dict()

                remote_group = mso_values.get("destination", {}).get("remote")
                if remote_group:
                    mso_update_values[("destination", "remote", "epgRef")] = remote_group.get("epgRef")
                    mso_update_values[("destination", "remote", "spanVersion")] = remote_group.get("spanVersion")
                    mso_update_values[("destination", "remote", "enforceSpanVersion")] = remote_group.get("enforceSpanVersion")
                    mso_update_values[("destination", "remote", "destIPAddress")] = remote_group.get("destIPAddress")
                    mso_update_values[("destination", "remote", "srcIPPrefix")] = remote_group.get("srcIPPrefix")
                    mso_update_values[("destination", "remote", "flowID")] = remote_group.get("flowID")
                    mso_update_values[("destination", "remote", "ttl")] = remote_group.get("ttl")
                    mso_update_values[("destination", "remote", "dscp")] = remote_group.get("dscp")

            if destination_port:
                if match.details.get("destination", {}).get("remote", {}).get("epgRef"):
                    mso_update_values[("destination", "remote")] = dict()

                if match.details.get("destination", {}).get("local", {}).get("portChannel"):
                    mso_update_values[("destination", "local")] = dict()

                mso_update_values[("destination", "local")] = dict(accessInterface=destination_port_uuid)

            if destination_port_channel:
                if match.details.get("destination", {}).get("remote", {}).get("epgRef"):
                    mso_update_values[("destination", "remote")] = dict()

                if match.details.get("destination", {}).get("local", {}).get("accessInterface"):
                    mso_update_values[("destination", "local")] = dict()

                mso_update_values[("destination", "local")] = dict(portChannel=destination_port_channel_uuid)

            proposed_payload = copy.deepcopy(match.details)
            append_update_ops_data(ops, proposed_payload, span_session_path, mso_update_values)
            mso.sanitize(proposed_payload, collate=True)
        else:
            mso_values["name"] = name
            mso_values["description"] = description

            if admin_state is not None:
                mso_values["sourceGroup"] = dict(enableAdminState=admin_state)

            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=span_session_path, value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=span_session_path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            response.get("monitoringTemplate", {}).get("template", {}).get("spanSessions", []),
            [KVPair("uuid", uuid) if uuid else KVPair("name", name)],
        )
        if match:
            set_fabric_span_session_object_details(mso_template, site_id, match.details)
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        set_fabric_span_session_object_details(mso_template, site_id, mso.proposed)
        mso.existing = mso.proposed if state == "present" else {}
    mso.exit_json()


def set_fabric_span_session_object_details(mso_template, site_id, span_session):
    if span_session:
        span_session.update({"templateId": mso_template.template_id, "templateName": mso_template.template_name})
        if span_session.get("destination", {}).get("local", {}).get("accessInterface"):
            interface = mso_template.mso.get_site_interface_details(site_id, span_session.get("destination").get("local").get("accessInterface"))
            interface.pop("uuid", None)
            span_session.get("destination").get("local").update(interface)
        else:
            reference_details = {
                "remote": {
                    "name": "epgName",
                    "reference": "epgRef",
                    "type": "epg",
                    "template": "epgTemplateName",
                    "templateId": "epgTemplateId",
                    "schema": "epgSchemaName",
                    "schemaId": "epgSchemaId",
                },
                "local": {
                    "name": "portChannelName",
                    "reference": "portChannel",
                    "type": "portChannel",
                    "template": "portChannelTemplateName",
                    "templateId": "portChannelTemplateId",
                },
            }
            mso_template.update_config_with_template_and_references(
                span_session.get("destination"),
                reference_details,
                False,
            )
    return span_session


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_log_exporter
short_description: Manages log-exporter objects on Checkpoint over Web Services API
description:
  - Manages log-exporter objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82 JHF management version.
version_added: "6.5.0"
author: "Dor Berenstein (@chkp-dorbe)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  target_server:
    description:
      - Target server port to which logs are exported.
    type: str
  target_port:
    description:
      - Port number of the target server.
    type: int
  protocol:
    description:
      - Protocol used to send logs to the target server.
    type: str
    choices: ['udp', 'tcp']
  enabled:
    description:
      - Indicates whether to enable export.
    type: bool
  attachments:
    description:
      - Log exporter attachments.
    type: dict
    suboptions:
      add_link_to_log_attachment:
        description:
          - Indicates whether to add link to log attachment in SmartView.
        type: bool
      add_link_to_log_details:
        description:
          - Indicates whether to add link to log details in SmartView.
        type: bool
      add_log_attachment_id:
        description:
          - Indicates whether to add log attachment ID.
        type: bool
  data_manipulation:
    description:
      - Log exporter data manipulation.
    type: dict
    suboptions:
      aggregate_log_updates:
        description:
          - Indicates whether to aggregate log updates.
        type: bool
      format:
        description:
          - Logs format.
        type: str
        choices: ['syslog', 'cef', 'leef', 'generic', 'splunk', 'logrhythm', 'json']
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-log-exporter
  cp_mgmt_log_exporter:
    attachments:
      add_link_to_log_attachment: true
    name: newLogExporter
    protocol: tcp
    state: present
    target_port: 1234
    target_server: 1.2.3.4

- name: set-log-exporter
  cp_mgmt_log_exporter:
    data_manipulation:
      format: json
    name: newLogExporter
    state: present
    target_port: 999

- name: delete-log-exporter
  cp_mgmt_log_exporter:
    name: newLogExporter
    state: absent
"""

RETURN = """
cp_mgmt_log_exporter:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        target_server=dict(type='str'),
        target_port=dict(type='int'),
        protocol=dict(type='str', choices=['udp', 'tcp']),
        enabled=dict(type='bool'),
        attachments=dict(type='dict', options=dict(
            add_link_to_log_attachment=dict(type='bool'),
            add_link_to_log_details=dict(type='bool'),
            add_log_attachment_id=dict(type='bool')
        )),
        data_manipulation=dict(type='dict', options=dict(
            aggregate_log_updates=dict(type='bool'),
            format=dict(type='str', choices=['syslog', 'cef', 'leef', 'generic', 'splunk', 'logrhythm', 'json'])
        )),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements="str"),
        tags=dict(type='list', elements="str"),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'log-exporter'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Gaspard Micol (@gmicol) <gmicol@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_match_rule_prefix
version_added: "2.12.0"
short_description: Manage Match Prefix List on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Match Prefix List inside a Match Rule Policy on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v4.1 (NDO v5.1) and later.
author:
- Gaspard Micol (@gmicol)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Tenant Policy template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the template.
    - The template must be a Tenant Policy template.
    - This parameter or O(template) is required.
    type: str
  match_rule_policy:
    description:
    - The name of the Match Rule Policy.
    - This parameter or O(match_rule_policy_uuid) is required.
    type: str
  match_rule_policy_uuid:
    description:
    - The UUID of the Match Rule Policy.
    - This parameter or O(match_rule_policy) is required.
    type: str
  prefix:
    description:
    - The Prefix IP to match.
    type: str
  description:
    description:
    - The description of the Match Prefix.
    type: str
  aggregate:
    description:
    - The aggregate flag enabling route aggregation.
    - Defaults to C(false) when unset during creation.
    type: bool
  from_prefix:
    description:
    - The subnet value from which to aggregate.
    - The value must be below O(to_prefix) and above the subnet of the Prefix IP.
    - The value must be between 0 and 32.
    - Defaults to C(0) when unset during creation.
    type: int
    aliases: [ from ]
  to_prefix:
    description:
    - The subnet value to which to aggregate.
    - The value must be above O(from_prefix) and above the subnet of the Prefix IP.
    - The value must be between 0 and 32.
    - Defaults to C(0) when unset during creation.
    type: int
    aliases: [ to ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant Policy template.
- The O(match_rule_policy) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_match_rule_policy) to create the Match Rule Policy.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_match_rule_policy
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a Match Prefix
  cisco.mso.ndo_match_rule_prefix:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    match_rule_policy: ansible_match_rule_policy
    prefix: 10.10.10.1/24
    description: Ansible Match Prefix
    aggregate: false
    from_prefix: 0
    to_prefix: 0
    state: present
  register: create_match_prefix

- name: Update a Match Prefix using Match Rule Policy's name
  cisco.mso.ndo_match_rule_prefix:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    match_rule_policy: ansible_match_rule_policy
    prefix: 10.10.10.1/24
    description: Updated Ansible Match Prefix
    aggregate: true
    from_prefix: 25
    to_prefix: 31
    state: present

- name: Update a Match Prefix using Match Rule Policy's UUID
  cisco.mso.ndo_match_rule_prefix:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    match_rule_policy_uuid: "{{ create_match_prefix.current.matchRulePolicyUuid }}"
    prefix: 10.10.10.1/24
    description: Updated Ansible Match Prefix
    aggregate: true
    from_prefix: 25
    to_prefix: 31
    state: present

- name: Query a Match Prefix using Match Rule Policy's name
  cisco.mso.ndo_match_rule_prefix:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    match_rule_policy: ansible_match_rule_policy
    prefix: 10.10.10.1/24
    state: query
  register: query_with_name

- name: Query a Match Prefix using  Match Rule Policy's UUID
  cisco.mso.ndo_match_rule_prefix:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    match_rule_policy_uuid: "{{ create_match_prefix.current.matchRulePolicyUuid }}"
    prefix: 10.10.10.1/24
    state: query
  register: query_with_uuid

- name: Query all Match Prefixes in a Match Rule Policy
  cisco.mso.ndo_match_rule_prefix:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    match_rule_policy: ansible_match_rule_policy
    state: query
  register: query_all_objects

- name: Delete a Match Prefix using Match Rule Policy's name
  cisco.mso.ndo_match_rule_prefix:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    match_rule_policy: ansible_match_rule_policy
    prefix: 10.10.10.1/24
    state: absent

- name: Delete a Match Prefix using Match Rule Policy's UUID
  cisco.mso.ndo_match_rule_prefix:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template_id: ansible_test
    match_rule_policy_uuid: "{{ create_match_prefix.current.matchRulePolicyUuid }}"
    prefix: 10.10.10.1/24
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.templates import MSOTemplates
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        match_rule_policy=dict(type="str"),
        match_rule_policy_uuid=dict(type="str"),
        prefix=dict(type="str"),
        description=dict(type="str"),
        aggregate=dict(type="bool"),
        from_prefix=dict(type="int", aliases=["from"]),
        to_prefix=dict(type="int", aliases=["to"]),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ("template", "template_id"),
            ["match_rule_policy", "match_rule_policy_uuid"],
        ],
        required_if=[
            ["state", "absent", ["prefix"]],
            ["state", "present", ["prefix"]],
        ],
        required_one_of=[
            ["template", "template_id"],
            ["match_rule_policy", "match_rule_policy_uuid"],
        ],
    )

    mso = MSOModule(module)
    mso_templates = MSOTemplates(mso)

    template_name = module.params.get("template")
    template_id = module.params.get("template_id")
    match_rule_policy = module.params.get("match_rule_policy")
    match_rule_policy_uuid = module.params.get("match_rule_policy_uuid")
    prefix = module.params.get("prefix")
    description = module.params.get("description")
    aggregate = module.params.get("aggregate")
    from_prefix = module.params.get("from_prefix")
    to_prefix = module.params.get("to_prefix")
    state = module.params.get("state")

    mso_template = mso_templates.get_template("tenant", template_name, template_id)
    mso_template.validate_template("tenantPolicy")

    match_rule_policy_object = mso_template.get_match_rule_policy_object(match_rule_policy_uuid, match_rule_policy)
    if not match_rule_policy_object:
        mso.fail_json(
            msg="The Match Rule Policy with the following {0[0]} '{0[1]}' does not exist".format(
                ("UUID", match_rule_policy_uuid) if match_rule_policy_uuid else ("name", match_rule_policy)
            )
        )
    match_identifiers = {"prefix": prefix}
    match = mso_template.get_direct_child_object(match_rule_policy_object, "Match Prefix", "matchPrefixList", match_identifiers)

    if prefix and match:
        mso.existing = mso.previous = copy.deepcopy(
            mso_template.update_match_rule_policy_child_object_with_template_and_parent(match_rule_policy_object.details, match.details)
        )  # Query a specific object
    elif match:
        mso.existing = [
            mso_template.update_match_rule_policy_child_object_with_template_and_parent(match_rule_policy_object.details, obj) for obj in match
        ]  # Query all objects

    match_prefix_path = "/tenantPolicyTemplate/template/matchRulePolicies/{0}/matchPrefixList/{1}".format(
        match_rule_policy_object.index,
        match.index if match else "-",
    )

    ops = []

    if state == "present":

        mso_values = {
            "prefix": prefix,
            "aggregate": aggregate,
            "description": description,
            "fromPfxLen": from_prefix,
            "toPfxLen": to_prefix,
        }

        if match:
            append_update_ops_data(ops, match.details, match_prefix_path, mso_values)
            mso.sanitize(mso_values, collate=True)
        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=match_prefix_path, value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=match_prefix_path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match_rule_policy_object = mso_template.get_match_rule_policy_object(match_rule_policy_uuid, match_rule_policy, search_object=response)
        match = mso_template.get_direct_child_object(match_rule_policy_object, "Match Prefix", "matchPrefixList", match_identifiers)
        if match:
            mso.existing = mso_template.update_match_rule_policy_child_object_with_template_and_parent(
                match_rule_policy_object.details, match.details
            )  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        if mso.proposed:
            mso_template.update_match_rule_policy_child_object_with_template_and_parent(match_rule_policy_object.details, mso.proposed)
        mso.existing = mso.proposed if state == "present" else {}
    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2017, Joris Weijters <joris.weijters@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Joris Weijters (@molekuul)
module: aix_inittab
short_description: Manages the C(inittab) on AIX
description:
  - Manages the C(inittab) on AIX.
deprecated:
  removed_in: 15.0.0
  why: The module is not actively maintained.
  alternative: >-
    Use C(ibm.power_aix.inittab) instead.
    See U(https://ibm.github.io/ansible-power-aix/modules/inittab.html) for details.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of the C(inittab) entry.
    type: str
    required: true
    aliases: [service]
  runlevel:
    description:
      - Runlevel of the entry.
    type: str
    required: true
  action:
    description:
      - Action the C(init) process performs for this entry.
    type: str
    choices:
      - boot
      - bootwait
      - hold
      - initdefault
      - 'off'
      - once
      - ondemand
      - powerfail
      - powerwait
      - respawn
      - sysinit
      - wait
  command:
    description:
      - Command to be executed.
    type: str
    required: true
  insertafter:
    description:
      - After which C(inittab) line should the new entry inserted.
    type: str
  state:
    description:
      - Whether the entry should be present or absent in the C(inittab) file.
    type: str
    choices: [absent, present]
    default: present
notes:
  - The changes are persistent across reboots.
  - You need root rights to read or adjust the C(inittab) with the C(lsitab), C(chitab), C(mkitab) or C(rmitab) commands.
  - Tested on AIX 7.1.
"""

EXAMPLES = r"""
# Add service startmyservice to the inittab, directly after service existingservice.
- name: Add startmyservice to inittab
  community.general.aix_inittab:
    name: startmyservice
    runlevel: 4
    action: once
    command: echo hello
    insertafter: existingservice
    state: present
  become: true

# Change inittab entry startmyservice to runlevel "2" and processaction "wait".
- name: Change startmyservice to inittab
  community.general.aix_inittab:
    name: startmyservice
    runlevel: 2
    action: wait
    command: echo hello
    state: present
  become: true

- name: Remove startmyservice from inittab
  community.general.aix_inittab:
    name: startmyservice
    runlevel: 2
    action: wait
    command: echo hello
    state: absent
  become: true
"""

RETURN = r"""
name:
  description: Name of the adjusted C(inittab) entry.
  returned: always
  type: str
  sample: startmyservice
"""


from ansible.module_utils.basic import AnsibleModule


def check_current_entry(module):
    # Check if entry exists, if not return False in exists in return dict,
    # if true return True and the entry in return dict
    existsdict = {"exist": False}
    lsitab = module.get_bin_path("lsitab")
    (rc, out, err) = module.run_command([lsitab, module.params["name"]])
    if rc == 0:
        keys = ("name", "runlevel", "action", "command")
        values = out.split(":")
        # strip non readable characters as \n
        values = map(lambda s: s.strip(), values)
        existsdict = dict(zip(keys, values))
        existsdict.update({"exist": True})
    return existsdict


def main():
    # initialize
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str", required=True, aliases=["service"]),
            runlevel=dict(type="str", required=True),
            action=dict(
                type="str",
                choices=[
                    "boot",
                    "bootwait",
                    "hold",
                    "initdefault",
                    "off",
                    "once",
                    "ondemand",
                    "powerfail",
                    "powerwait",
                    "respawn",
                    "sysinit",
                    "wait",
                ],
            ),
            command=dict(type="str", required=True),
            insertafter=dict(type="str"),
            state=dict(type="str", default="present", choices=["absent", "present"]),
        ),
        supports_check_mode=True,
    )

    result = {"name": module.params["name"], "changed": False, "msg": ""}

    # Find commandline strings
    mkitab = module.get_bin_path("mkitab")
    rmitab = module.get_bin_path("rmitab")
    chitab = module.get_bin_path("chitab")
    rc = 0
    err = None

    # check if the new entry exists
    current_entry = check_current_entry(module)

    # if action is install or change,
    if module.params["state"] == "present":
        # create new entry string
        new_entry = (
            f"{module.params['name']}:{module.params['runlevel']}:{module.params['action']}:{module.params['command']}"
        )

        # If current entry exists or fields are different(if the entry does not
        # exists, then the entry will be created
        if (not current_entry["exist"]) or (
            module.params["runlevel"] != current_entry["runlevel"]
            or module.params["action"] != current_entry["action"]
            or module.params["command"] != current_entry["command"]
        ):
            # If the entry does exist then change the entry
            if current_entry["exist"]:
                if not module.check_mode:
                    (rc, out, err) = module.run_command([chitab, new_entry])
                if rc != 0:
                    module.fail_json(msg="could not change inittab", rc=rc, err=err)
                result["msg"] = f"changed inittab entry {current_entry['name']}"
                result["changed"] = True

            # If the entry does not exist create the entry
            elif not current_entry["exist"]:
                if module.params["insertafter"]:
                    if not module.check_mode:
                        (rc, out, err) = module.run_command([mkitab, "-i", module.params["insertafter"], new_entry])
                else:
                    if not module.check_mode:
                        (rc, out, err) = module.run_command([mkitab, new_entry])

                if rc != 0:
                    module.fail_json(msg="could not adjust inittab", rc=rc, err=err)
                result["msg"] = f"add inittab entry {module.params['name']}"
                result["changed"] = True

    elif module.params["state"] == "absent":
        # If the action is remove and the entry exists then remove the entry
        if current_entry["exist"]:
            if not module.check_mode:
                (rc, out, err) = module.run_command([rmitab, module.params["name"]])
                if rc != 0:
                    module.fail_json(msg="could not remove entry from inittab)", rc=rc, err=err)
            result["msg"] = f"removed inittab entry {current_entry['name']}"
            result["changed"] = True

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# (c) 2020, Alexei Znamensky <russoz@gmail.com>
# Copyright (c) 2020, Ansible Project
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible.module_utils.basic import AnsibleModule


class DeprecateAttrsMixin:
    def _deprecate_setup(
        self, attr: str, target: object | None, module: AnsibleModule | None
    ) -> tuple[object, AnsibleModule, dict[str, t.Any], dict[str, t.Any]]:
        if target is None:
            target = self
        if not hasattr(target, attr):
            raise ValueError(f"Target {target} has no attribute {attr}")
        if module is None:
            if isinstance(target, AnsibleModule):
                module = target
            elif hasattr(target, "module") and isinstance(target.module, AnsibleModule):
                module = target.module
            else:
                raise ValueError(
                    "Failed to automatically discover the AnsibleModule instance. Pass 'module' parameter explicitly."
                )

        # setup internal state dicts
        value_attr = "__deprecated_attr_value"
        trigger_attr = "__deprecated_attr_trigger"
        if not hasattr(target, value_attr):
            setattr(target, value_attr, {})
        if not hasattr(target, trigger_attr):
            setattr(target, trigger_attr, {})
        value_dict = getattr(target, value_attr)
        trigger_dict = getattr(target, trigger_attr)
        return target, module, value_dict, trigger_dict

    def _deprecate_attr(
        self,
        attr: str,
        msg: str,
        version: str | None = None,
        date: str | None = None,
        collection_name: str | None = None,
        target: object | None = None,
        value=None,
        module: AnsibleModule | None = None,
    ) -> None:
        target, module, value_dict, trigger_dict = self._deprecate_setup(attr, target, module)

        value_dict[attr] = getattr(target, attr, value)
        trigger_dict[attr] = False

        def _trigger():
            if not trigger_dict[attr]:
                module.deprecate(msg, version=version, date=date, collection_name=collection_name)
                trigger_dict[attr] = True

        def _getter(_self):
            _trigger()
            return value_dict[attr]

        def _setter(_self, new_value):
            _trigger()
            value_dict[attr] = new_value

        # override attribute
        prop = property(_getter)
        setattr(target, attr, prop)
        setattr(target, f"_{attr}_setter", prop.setter(_setter))



#!/usr/bin/python

# Copyright (c) 2016, Kamil Szczygiel <kamil.szczygiel () intel.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: influxdb_database
short_description: Manage InfluxDB databases
description:
  - Manage InfluxDB databases.
author: "Kamil Szczygiel (@kamsz)"
requirements:
  - "influxdb >= 0.9"
  - requests
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  database_name:
    description:
      - Name of the database.
    required: true
    type: str
  state:
    description:
      - Determines if the database should be created or destroyed.
    choices: [absent, present]
    default: present
    type: str
extends_documentation_fragment:
  - community.general._influxdb
  - community.general._attributes
"""

EXAMPLES = r"""
# Example influxdb_database command from Ansible Playbooks
- name: Create database
  community.general.influxdb_database:
    hostname: "{{influxdb_ip_address}}"
    database_name: "{{influxdb_database_name}}"

- name: Destroy database
  community.general.influxdb_database:
    hostname: "{{influxdb_ip_address}}"
    database_name: "{{influxdb_database_name}}"
    state: absent

- name: Create database using custom credentials
  community.general.influxdb_database:
    hostname: "{{influxdb_ip_address}}"
    username: "{{influxdb_username}}"
    password: "{{influxdb_password}}"
    database_name: "{{influxdb_database_name}}"
    ssl: true
    validate_certs: true
"""

RETURN = r"""
# only defaults
"""

try:
    import requests.exceptions
    from influxdb import exceptions
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._influxdb import InfluxDb


def find_database(module, client, database_name):
    database = None

    try:
        databases = client.get_list_database()
        for db in databases:
            if db["name"] == database_name:
                database = db
                break
    except requests.exceptions.ConnectionError as e:
        module.fail_json(msg=str(e))
    return database


def create_database(module, client, database_name):
    if not module.check_mode:
        try:
            client.create_database(database_name)
        except requests.exceptions.ConnectionError as e:
            module.fail_json(msg=str(e))

    module.exit_json(changed=True)


def drop_database(module, client, database_name):
    if not module.check_mode:
        try:
            client.drop_database(database_name)
        except exceptions.InfluxDBClientError as e:
            module.fail_json(msg=e.content)

    module.exit_json(changed=True)


def main():
    argument_spec = InfluxDb.influxdb_argument_spec()
    argument_spec.update(
        database_name=dict(required=True, type="str"),
        state=dict(default="present", type="str", choices=["present", "absent"]),
    )
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    state = module.params["state"]

    influxdb = InfluxDb(module)
    client = influxdb.connect_to_influxdb()
    database_name = influxdb.database_name
    database = find_database(module, client, database_name)

    if state == "present":
        if database:
            module.exit_json(changed=False)
        else:
            create_database(module, client, database_name)

    if state == "absent":
        if database:
            drop_database(module, client, database_name)
        else:
            module.exit_json(changed=False)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_mapped_drive
short_description: Map network drives for users
description:
- Allows you to modify mapped network drives for individual users.
- Also support WebDAV endpoints in the UNC form.
- While this module can manage mapped drives, you will not be able to use those
  mapped drives in any subsequent Ansible task. Mapped drives are only visible
  on interactive logons like through RDP and not network logons like WinRM or
  SSH that Ansible uses to invoke tasks.
version_added: 2.7.0
options:
  letter:
    description:
    - The letter of the network path to map to.
    - This letter must not already be in use with Windows.
    type: str
    required: yes
  password:
    description:
    - The password for C(username) that is used when testing the initial
      connection.
    - This is never saved with a mapped drive, use the M(ansible.windows.win_credential) module
      to persist a username and password for a host.
    type: str
  path:
    description:
    - The UNC path to map the drive to.
    - If pointing to a WebDAV location this must still be in a UNC path in the
      format C(\\hostname\path) and not a URL, see examples for more details.
    - To specify a C(https) WebDAV path, add C(@SSL) after the hostname. To
      specify a custom WebDAV port add C(@<port num>) after the C(@SSL) or
      hostname portion of the UNC path, e.g. C(\\server@SSL@1234) or
      C(\\server@1234).
    - This is required if C(state=present).
    - If C(state=absent) and I(path) is not set, the module will delete the
      mapped drive regardless of the target.
    - If C(state=absent) and the I(path) is set, the module will throw an error
      if path does not match the target of the mapped drive.
    type: path
  state:
    description:
    - If C(present) will ensure the mapped drive exists.
    - If C(absent) will ensure the mapped drive does not exist.
    type: str
    choices: [ absent, present ]
    default: present
  username:
    description:
    - The username that is used when testing the initial connection.
    - This is never saved with a mapped drive, the M(ansible.windows.win_credential) module
      to persist a username and password for a host.
    - This is required if the mapped drive requires authentication with
      custom credentials and become, or CredSSP cannot be used.
    - If become or CredSSP is used, any credentials saved with
      M(ansible.windows.win_credential) will automatically be used instead.
    type: str
notes:
- You cannot use this module to access a mapped drive in another Ansible task,
  drives mapped with this module are only accessible when logging in
  interactively with the user through the console or RDP.
- It is recommend to run this module with become or CredSSP when the remote
  path requires authentication.
- When using become or CredSSP, the task will have access to any local
  credentials stored in the user's vault.
- If become or CredSSP is not available, the I(username) and I(password)
  options can be used for the initial authentication but these are not
  persisted.
- WebDAV paths must have the WebDAV client feature installed for this module to
  map those paths. This is installed by default on desktop Windows editions but
  Windows Server hosts need to install the C(WebDAV-Redirector) feature using
  M(ansible.windows.win_feature).
seealso:
- module: ansible.windows.win_credential
author:
- Jordan Borean (@jborean93)
'''

EXAMPLES = r'''
- name: Create a mapped drive under Z
  ansible.windows.win_mapped_drive:
    letter: Z
    path: \\domain\appdata\accounting

- name: Delete any mapped drives under Z
  ansible.windows.win_mapped_drive:
    letter: Z
    state: absent

- name: Only delete the mapped drive Z if the paths match (error is thrown otherwise)
  ansible.windows.win_mapped_drive:
    letter: Z
    path: \\domain\appdata\accounting
    state: absent

- name: Create mapped drive with credentials and save the username and password
  block:
    - name: Save the network credentials required for the mapped drive
      ansible.windows.win_credential:
        name: server
        type: domain_password
        username: username@DOMAIN
        secret: Password01
        state: present

    - name: Create a mapped drive that requires authentication
      ansible.windows.win_mapped_drive:
        letter: M
        path: \\SERVER\C$
        state: present
  vars:
    # become is required to save and retrieve the credentials in the tasks
    ansible_become: true
    ansible_become_method: runas
    ansible_become_user: '{{ ansible_user }}'
    ansible_become_pass: '{{ ansible_password }}'

- name: Create mapped drive with credentials that do not persist on the next logon
  ansible.windows.win_mapped_drive:
    letter: M
    path: \\SERVER\C$
    state: present
    username: '{{ ansible_user }}'
    password: '{{ ansible_password }}'

# This should only be required for Windows Server OS'
- name: Ensure WebDAV client feature is installed
  ansible.windows.win_feature:
    name: WebDAV-Redirector
    state: present
  register: webdav_feature

- name: Reboot after installing WebDAV client feature
  ansible.windows.win_reboot:
  when: webdav_feature.reboot_required

- name: Map the HTTPS WebDAV location
  ansible.windows.win_mapped_drive:
    letter: W
    path: \\live.sysinternals.com@SSL\tools  # https://live.sysinternals.com/tools
    state: present
'''

RETURN = r'''
'''



# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:

    BULK_OPERATIONS = r"""
options:
  bulk_operation_threshold:
    description:
      - Determines the threshold from when on bulk operations are used.
      - The default value 2 means that if 2 or more operations of a kind are planned,
        and the API supports bulk operations for this kind of operation, they will
        be used.
    type: int
    default: 2
"""

    RECORD_TRANSFORMATION = r"""
options:
  txt_transformation:
    description:
      - Determines how TXT entry values are converted between the API and this module's
        input and output.
      - The value V(api) means that values are returned from this module as they are returned
        from the API, and pushed to the API as they have been passed to this module. For
        idempotency checks, the input string will be compared to the strings returned by the
        API. The API might automatically transform some values, like splitting long values or
        adding quotes, which can cause problems with idempotency.
      - The value V(unquoted) automatically transforms values so that you can pass in unquoted
        values, and the module will return unquoted values. If you pass in quoted values, they
        will be double-quoted.
      - The value V(quoted) automatically transforms values so that you must use quoting for values
        that contain spaces, characters such as quotation marks and backslashes, and that are
        longer than 255 bytes. It also makes sure to return values from the API in a normalized
        encoding.
      - The default value, V(unquoted), ensures that you can work with values without having
        to care about how to correctly quote for DNS. Most users should use one of V(unquoted)
        or V(quoted), but not V(api).
      - B(Note:) the conversion code assumes UTF-8 encoding for values. If you need another
        encoding use O(txt_transformation=api) and handle the encoding yourself.
    type: str
    choices:
      - api
      - quoted
      - unquoted
    default: unquoted
  txt_character_encoding:
    description:
      - Whether to treat numeric escape sequences (V(\\xyz)) as octal or decimal numbers.
        This is only used when O(txt_transformation=quoted).
      - The default changed to V(decimal) in community.dns 3.0.0. Before, the default used to be V(octal).
        The value V(decimal) is compatible to L(RFC 1035, https://www.ietf.org/rfc/rfc1035.txt).
    type: str
    choices:
      - decimal
      - octal
    default: decimal
    version_added: 2.5.0
"""



#!/usr/bin/python
#
# Copyright (c) 2021 Praveen Ghuge(@praveenghuge) Karl Dasan(@karldas30) Saurabh Malpani (@saurabh3796)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_eventhub_info
version_added: "1.6.0"
short_description: Get Azure Event Hub
description:
    - Get facts of Azure Event Hub.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    namespace_name:
        description:
            - The name of the namspace.
        required: False
        type: str
    name:
        description:
            - The name of the Event hub.
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - Saurabh Malpani (@saurabh3796)
'''


EXAMPLES = '''
- name: Get facts of specific Event hub
  azure.azcollection.azure_rm_eventhub_info:
    resource_group: myResourceGroup
    name: myEventHub
'''

RETURN = '''
state:
    description:
        - Current state of the Event Hub namesapce or Event Hub.
    returned: always
    type: dict
    sample: {
            "additional_properties": {},
            "created_at": "2021-04-19T12:49:46.597Z",
            "critical": null,
            "data_center": null,
            "status": "Active",
            "location": "eastus",
            "metric_id": "149f0952-6f3d-48ba-9e98-57011575cbbd:eventhubtestns1753",
            "name": "eventhubtestns1753",
            "namespace_type": null,
            "provisioning_state": "Succeeded",
            "region": null,
            "scale_unit": null,
            "service_bus_endpoint": "https://eventhubtestns1753.servicebus.windows.net:443/",
            "sku": "Basic",
            "tags": {},
            "type": "Microsoft.EventHub/Namespaces",
            "updated_at": "2021-04-19T12:54:33.397Z"
    }
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMEventHubInfo(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            namespace_name=dict(
                type='str',
                required=False
            ),
            name=dict(
                type='str',
                required=False
            )
        )

        # store the results of the module operation
        self.results = dict(
            changed=False)
        self.resource_group = None
        self.namespace_name = None
        self.name = None
        self.tags = None

        super(AzureRMEventHubInfo, self).__init__(
            self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name and self.namespace_name and self.resource_group:
            results = self.get_event_hub()
            self.results['eventhub'] = [
                self.event_hub_to_dict(x) for x in results]
        elif self.namespace_name:
            results = self.get_namespace()
            self.results['namespace'] = [
                self.namespace_to_dict(x) for x in results]
        elif self.name is None and self.namespace_name is None:
            results = self.list_all_namespace()
            self.results['namespaces'] = [
                self.namespace_to_dict(x) for x in results]
        return self.results

    def get_namespace(self):
        '''
        Get the namespace using resource group and namespace name
        '''
        response = None
        results = []
        try:
            response = self.event_hub_client.namespaces.get(
                self.resource_group, self.namespace_name)
            self.log("Response : {0}".format(response))

        except ResourceNotFoundError as e:
            self.fail('Could not get info for namespace. {0}').format(
                str(e))

        if response:
            results = [response]
        return results

    def get_event_hub(self):
        '''
        get event hub using resource_group, namespace_name and name.
        '''
        response = None
        results = []

        try:
            response = self.event_hub_client.event_hubs.get(
                self.resource_group, self.namespace_name, self.name)

        except ResourceNotFoundError as e:
            self.fail('Could not get info for event hub. {0}').format(
                str(e))

        if response:
            results = [response]
        return results

    def list_all_namespace(self):
        '''
        List all namespaces in particular resource_group
        '''
        self.log('List items for resource group')
        try:
            response = self.event_hub_client.namespaces.list_by_resource_group(
                self.resource_group)

        except Exception as exc:
            self.fail(
                "Failed to list for resource group {0} - {1}".format(self.resource_group, str(exc)))

        results = []
        for item in response:
            results.append(item)
        return results

    def namespace_to_dict(self, item):
        # turn event hub object into a dictionary (serialization)
        namespace = item.as_dict()
        result = dict(
            additional_properties=namespace.get(
                'additional_properties', {}),
            name=namespace.get('name', None),
            type=namespace.get('type', None),
            location=namespace.get(
                'location', '').replace(' ', '').lower(),
            sku=namespace.get("sku").get("name"),
            tags=namespace.get('tags', None),
            provisioning_state=namespace.get(
                'provisioning_state', None),
            region=namespace.get('region', None),
            metric_id=namespace.get('metric_id', None),
            service_bus_endpoint=namespace.get(
                'service_bus_endpoint', None),
            scale_unit=namespace.get('scale_unit', None),
            enabled=namespace.get('enabled', None),
            critical=namespace.get('critical', None),
            data_center=namespace.get('data_center', None),
            namespace_type=namespace.get('namespace_type', None),
            updated_at=namespace.get('updated_at', None),
            created_at=namespace.get('created_at', None),
            is_auto_inflate_enabled=namespace.get(
                'is_auto_inflate_enabled', None),
            maximum_throughput_units=namespace.get(
                'maximum_throughput_units', None),
            identity=namespace.get('identity', None)
        )
        return result

    def event_hub_to_dict(self, item):
        # turn event hub object into a dictionary (serialization)
        event_hub = item.as_dict()
        result = dict()
        if item.additional_properties:
            result['additional_properties'] = item.additional_properties
        result['name'] = event_hub.get('name', None)
        result['partition_ids'] = event_hub.get('partition_ids', None)
        result['created_at'] = event_hub.get('created_at', None)
        result['updated_at'] = event_hub.get('updated_at', None)
        result['message_retention_in_days'] = event_hub.get(
            'message_retention_in_days', None)
        result['partition_count'] = event_hub.get('partition_count', None)
        result['status'] = event_hub.get('status', None)
        result['tags'] = event_hub.get('tags', None)
        result['identity'] = event_hub.get('identity', None)
        return result


def main():
    AzureRMEventHubInfo()


if __name__ == '__main__':
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import logging
import os
import signal
import socket
import subprocess
import traceback
import typing as t
from queue import Empty
from urllib.parse import urlparse

from .. import constants
from .._import_helper import HTTPAdapter, urllib3, urllib3_connection
from .basehttpadapter import BaseHTTPAdapter

PARAMIKO_IMPORT_ERROR: str | None  # pylint: disable=invalid-name
try:
    import paramiko
except ImportError:
    PARAMIKO_IMPORT_ERROR = traceback.format_exc()  # pylint: disable=invalid-name
else:
    PARAMIKO_IMPORT_ERROR = None  # pylint: disable=invalid-name

if t.TYPE_CHECKING:
    from collections.abc import Buffer, Mapping


RecentlyUsedContainer = urllib3._collections.RecentlyUsedContainer


class SSHSocket(socket.socket):
    def __init__(self, host: str) -> None:
        super().__init__(socket.AF_INET, socket.SOCK_STREAM)
        self.host = host
        self.port = None
        self.user = None
        if ":" in self.host:
            self.host, self.port = self.host.split(":")
        if "@" in self.host:
            self.user, self.host = self.host.split("@")

        self.proc: subprocess.Popen | None = None

    def connect(self, *args_: t.Any, **kwargs: t.Any) -> None:
        args = ["ssh"]
        if self.user:
            args = args + ["-l", self.user]

        if self.port:
            args = args + ["-p", self.port]

        args = args + ["--", self.host, "docker system dial-stdio"]

        preexec_func = None
        if not constants.IS_WINDOWS_PLATFORM:

            def f() -> None:
                signal.signal(signal.SIGINT, signal.SIG_IGN)

            preexec_func = f

        env = dict(os.environ)

        # drop LD_LIBRARY_PATH and SSL_CERT_FILE
        env.pop("LD_LIBRARY_PATH", None)
        env.pop("SSL_CERT_FILE", None)

        self.proc = subprocess.Popen(  # pylint: disable=consider-using-with
            args,
            env=env,
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
            preexec_fn=preexec_func,
        )

    def _write(self, data: Buffer) -> int:
        if not self.proc:
            raise RuntimeError(
                "SSH subprocess not initiated. connect() must be called first."
            )
        assert self.proc.stdin is not None
        if self.proc.stdin.closed:
            raise RuntimeError(
                "SSH subprocess not initiated. connect() must be called first after close()."
            )
        written = self.proc.stdin.write(data)
        self.proc.stdin.flush()
        return written

    def sendall(self, data: Buffer, *args: t.Any, **kwargs: t.Any) -> None:
        self._write(data)

    def send(self, data: Buffer, *args: t.Any, **kwargs: t.Any) -> int:
        return self._write(data)

    def recv(self, n: int, *args: t.Any, **kwargs: t.Any) -> bytes:
        if not self.proc:
            raise RuntimeError(
                "SSH subprocess not initiated. connect() must be called first."
            )
        assert self.proc.stdout is not None
        return self.proc.stdout.read(n)

    def makefile(self, mode: str, *args: t.Any, **kwargs: t.Any) -> t.IO:  # type: ignore
        if not self.proc:
            self.connect()
            assert self.proc is not None
        assert self.proc.stdout is not None
        self.proc.stdout.channel = self  # type: ignore

        return self.proc.stdout

    def close(self) -> None:
        if not self.proc:
            return
        assert self.proc.stdin is not None
        if self.proc.stdin.closed:
            return
        self.proc.stdin.write(b"\n\n")
        self.proc.stdin.flush()
        self.proc.terminate()


class SSHConnection(urllib3_connection.HTTPConnection):
    def __init__(
        self,
        *,
        ssh_transport: paramiko.Transport | None = None,
        timeout: int | float = 60,
        host: str,
    ) -> None:
        super().__init__("localhost", timeout=timeout)
        self.ssh_transport = ssh_transport
        self.timeout = timeout
        self.ssh_host = host
        self.sock: paramiko.Channel | SSHSocket | None = None

    def connect(self) -> None:
        if self.ssh_transport:
            channel = self.ssh_transport.open_session()
            channel.settimeout(self.timeout)
            channel.exec_command("docker system dial-stdio")
            self.sock = channel
        else:
            sock = SSHSocket(self.ssh_host)
            sock.settimeout(self.timeout)
            sock.connect()
            self.sock = sock


class SSHConnectionPool(urllib3.connectionpool.HTTPConnectionPool):
    scheme = "ssh"

    def __init__(
        self,
        *,
        ssh_client: paramiko.SSHClient | None = None,
        timeout: int | float = 60,
        maxsize: int = 10,
        host: str,
    ) -> None:
        super().__init__("localhost", timeout=timeout, maxsize=maxsize)
        self.ssh_transport: paramiko.Transport | None = None
        self.timeout = timeout
        if ssh_client:
            self.ssh_transport = ssh_client.get_transport()
        self.ssh_host = host

    def _new_conn(self) -> SSHConnection:
        return SSHConnection(
            ssh_transport=self.ssh_transport,
            timeout=self.timeout,
            host=self.ssh_host,
        )

    # When re-using connections, urllib3 calls fileno() on our
    # SSH channel instance, quickly overloading our fd limit. To avoid this,
    # we override _get_conn
    def _get_conn(self, timeout: int | float) -> SSHConnection:
        conn = None
        try:
            conn = self.pool.get(block=self.block, timeout=timeout)

        except AttributeError as exc:  # self.pool is None
            raise urllib3.exceptions.ClosedPoolError(self, "Pool is closed.") from exc

        except Empty as exc:
            if self.block:
                raise urllib3.exceptions.EmptyPoolError(
                    self,
                    "Pool reached maximum size and no more connections are allowed.",
                ) from exc
            # Oh well, we'll create a new connection then

        return conn or self._new_conn()


class SSHHTTPAdapter(BaseHTTPAdapter):
    __attrs__ = HTTPAdapter.__attrs__ + [
        "pools",
        "timeout",
        "ssh_client",
        "ssh_params",
        "max_pool_size",
    ]

    def __init__(
        self,
        base_url: str,
        timeout: int | float = 60,
        pool_connections: int = constants.DEFAULT_NUM_POOLS,
        max_pool_size: int = constants.DEFAULT_MAX_POOL_SIZE,
        shell_out: bool = False,
    ) -> None:
        self.ssh_client: paramiko.SSHClient | None = None
        if not shell_out:
            self._create_paramiko_client(base_url)
            self._connect()

        self.ssh_host = base_url
        if base_url.startswith("ssh://"):
            self.ssh_host = base_url[len("ssh://") :]

        self.timeout = timeout
        self.max_pool_size = max_pool_size
        self.pools = RecentlyUsedContainer(
            pool_connections, dispose_func=lambda p: p.close()
        )
        super().__init__()

    def _create_paramiko_client(self, base_url: str) -> None:
        logging.getLogger("paramiko").setLevel(logging.WARNING)
        self.ssh_client = paramiko.SSHClient()
        base_url_p = urlparse(base_url)
        assert base_url_p.hostname is not None
        self.ssh_params: dict[str, t.Any] = {
            "hostname": base_url_p.hostname,
            "port": base_url_p.port,
            "username": base_url_p.username,
        }
        ssh_config_file = os.path.expanduser("~/.ssh/config")
        if os.path.exists(ssh_config_file):
            conf = paramiko.SSHConfig()
            with open(ssh_config_file, "rt", encoding="utf-8") as f:
                conf.parse(f)
            host_config = conf.lookup(base_url_p.hostname)
            if "proxycommand" in host_config:
                self.ssh_params["sock"] = paramiko.ProxyCommand(
                    host_config["proxycommand"]
                )
            if "hostname" in host_config:
                self.ssh_params["hostname"] = host_config["hostname"]
            if base_url_p.port is None and "port" in host_config:
                self.ssh_params["port"] = host_config["port"]
            if base_url_p.username is None and "user" in host_config:
                self.ssh_params["username"] = host_config["user"]
            if "identityfile" in host_config:
                self.ssh_params["key_filename"] = host_config["identityfile"]

        self.ssh_client.load_system_host_keys()
        self.ssh_client.set_missing_host_key_policy(paramiko.RejectPolicy())

    def _connect(self) -> None:
        if self.ssh_client:
            self.ssh_client.connect(**self.ssh_params)

    def get_connection(
        self, url: str | bytes, proxies: Mapping[str, str] | None = None
    ) -> SSHConnectionPool:
        if not self.ssh_client:
            return SSHConnectionPool(
                ssh_client=self.ssh_client,
                timeout=self.timeout,
                maxsize=self.max_pool_size,
                host=self.ssh_host,
            )
        with self.pools.lock:
            pool = self.pools.get(url)
            if pool:
                return pool

            # Connection is closed try a reconnect
            if self.ssh_client and not self.ssh_client.get_transport():
                self._connect()

            pool = SSHConnectionPool(
                ssh_client=self.ssh_client,
                timeout=self.timeout,
                maxsize=self.max_pool_size,
                host=self.ssh_host,
            )
            self.pools[url] = pool

        return pool

    def close(self) -> None:
        super().close()
        if self.ssh_client:
            self.ssh_client.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_ipmi_over_lan_policy
short_description: IPMI over LAN Policy configuration for Cisco Intersight
description:
  - Manages IPMI over LAN Policy configuration on Cisco Intersight.
  - A policy to configure the IPMI over LAN settings on Cisco Intersight managed servers.
  - IPMI over LAN allows management of servers using the IPMI protocol over Ethernet networks.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/ipmioverlan/Policy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the IPMI over LAN Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the IPMI over LAN Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  enabled:
    description:
      - State of the IPMI Over LAN service on the endpoint.
      - Enable or disable IPMI over LAN functionality on the server.
    type: bool
    default: true
  privilege:
    description:
      - The highest privilege level that can be assigned to an IPMI session on a server.
      - This configuration is supported by all Standalone C-Series servers.
      - FI-attached C-Series servers with firmware at minimum of 4.2.3a support this configuration.
      - B/X-Series servers with firmware at minimum of 5.1.0.x support this configuration.
      - Privilege level 'user' is not supported for B/X-Series servers.
    type: str
    choices: ['admin', 'user', 'read-only']
    default: 'admin'
  encryption_key:
    description:
      - The encryption key to use for IPMI communication.
      - It should have an even number of hexadecimal characters and not exceed 40 characters.
      - Use "00" to disable encryption key use.
      - This configuration is supported by all Standalone C-Series servers.
      - FI-attached C-Series servers with firmware at minimum of 4.2.3a support this configuration.
      - B/X-Series servers with firmware at minimum of 5.1.0.x support this configuration.
      - IPMI commands using this key should append zeroes to the key to achieve a length of 40 characters.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create IPMI over LAN Policy with encryption enabled
  cisco.intersight.intersight_ipmi_over_lan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "IPMI-Policy-Encrypted"
    description: "IPMI over LAN policy with encryption"
    enabled: true
    privilege: "admin"
    encryption_key: "AB2134AC"
    tags:
      - Key: "Site"
        Value: "DataCenter-A"
    state: present

- name: Create IPMI over LAN Policy with encryption disabled
  cisco.intersight.intersight_ipmi_over_lan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "IPMI-Policy-No-Encryption"
    description: "IPMI over LAN policy without encryption"
    enabled: true
    privilege: "operator"
    encryption_key: "00"
    state: present

- name: Create disabled IPMI over LAN Policy
  cisco.intersight.intersight_ipmi_over_lan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "IPMI-Policy-Disabled"
    description: "Disabled IPMI over LAN policy"
    enabled: false
    state: present

- name: Delete IPMI over LAN Policy
  cisco.intersight.intersight_ipmi_over_lan_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "IPMI-Policy-Encrypted"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "IPMI-Policy-Encrypted",
        "ObjectType": "ipmioverlan.Policy",
        "Enabled": true,
        "Privilege": "admin",
        "IsEncryptionKeySet": true,
        "Tags": [
            {
                "Key": "Site",
                "Value": "DataCenter-A"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_encryption_key(module):
    """Validate the encryption key format"""
    encryption_key = module.params.get('encryption_key')
    if encryption_key and encryption_key != "00":
        # Check if it's a valid hex string
        try:
            int(encryption_key, 16)
        except ValueError:
            module.fail_json(msg="encryption_key must contain only hexadecimal characters")

        # Check if it has even number of characters
        if len(encryption_key) % 2 != 0:
            module.fail_json(msg="encryption_key must have an even number of characters")

        # Check if it doesn't exceed 40 characters
        if len(encryption_key) > 40:
            module.fail_json(msg="encryption_key must not exceed 40 characters")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enabled=dict(type='bool', default=True),
        privilege=dict(
            type='str',
            choices=['admin', 'user', 'read-only'],
            default='admin'
        ),
        encryption_key=dict(type='str', no_log=True)
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    # Validate encryption key format
    validate_encryption_key(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/ipmioverlan/Policies'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    # In case state is absent we don't want to populate the api_body with any values
    if intersight.module.params['state'] == 'present':
        intersight.api_body['Enabled'] = intersight.module.params['enabled']
        if intersight.module.params['enabled']:
            intersight.api_body['Privilege'] = intersight.module.params['privilege']
            # Add encryption key if provided
            if intersight.module.params.get('encryption_key'):
                intersight.api_body['EncryptionKey'] = intersight.module.params['encryption_key']

        intersight.set_tags_and_description()

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

DOCUMENTATION = r"""
---
module: s3_bucket_info
version_added: 1.0.0
version_added_collection: community.aws
author:
  - "Gerben Geijteman (@hyperized)"
short_description: Lists S3 buckets in AWS
description:
  - Lists S3 buckets and details about those buckets.
  - Prior to release 5.0.0 this module was called M(community.aws.aws_s3_bucket_info).
    The usage did not change.
options:
  name:
    description:
      - Name of bucket to query.
    type: str
    default: ""
    version_added: 1.4.0
  name_filter:
    description:
      - Limits buckets to only buckets who's name contain the string in O(name_filter).
    type: str
    default: ""
    version_added: 1.4.0
  bucket_facts:
    description:
      - Retrieve requested S3 bucket detailed information.
      - Each bucket_X option executes one API call, hence many options being set to V(true) will cause slower module execution.
      - You can limit buckets by using the O(name) or O(name_filter) option.
    suboptions:
      bucket_accelerate_configuration:
        description: Retrieve S3 accelerate configuration.
        type: bool
        default: False
      bucket_location:
        description: Retrieve S3 bucket location.
        type: bool
        default: False
      bucket_replication:
        description: Retrieve S3 bucket replication.
        type: bool
        default: False
      bucket_acl:
        description: Retrieve S3 bucket ACLs.
        type: bool
        default: False
      bucket_logging:
        description: Retrieve S3 bucket logging.
        type: bool
        default: False
      bucket_request_payment:
        description: Retrieve S3 bucket request payment.
        type: bool
        default: False
      bucket_tagging:
        description: Retrieve S3 bucket tagging.
        type: bool
        default: False
      bucket_cors:
        description: Retrieve S3 bucket CORS configuration.
        type: bool
        default: False
      bucket_notification_configuration:
        description: Retrieve S3 bucket notification configuration.
        type: bool
        default: False
      bucket_encryption:
        description: Retrieve S3 bucket encryption.
        type: bool
        default: False
      bucket_ownership_controls:
        description:
        - Retrieve S3 ownership controls.
        type: bool
        default: False
      bucket_website:
        description: Retrieve S3 bucket website.
        type: bool
        default: False
      bucket_policy:
        description: Retrieve S3 bucket policy.
        type: bool
        default: False
      bucket_policy_status:
        description: Retrieve S3 bucket policy status.
        type: bool
        default: False
      bucket_lifecycle_configuration:
        description: Retrieve S3 bucket lifecycle configuration.
        type: bool
        default: False
      public_access_block:
        description: Retrieve S3 bucket public access block.
        type: bool
        default: False
      bucket_versioning:
        description:
          - Retrieve the versioning state of a bucket.
          - To retrieve the versioning state of a bucket, you must be the bucket owner.
        type: bool
        default: False
        version_added: 7.3.0
    type: dict
    version_added: 1.4.0
  transform_location:
    description:
      - S3 bucket location for default us-east-1 is normally reported as C(null).
      - Setting this option to V(true) will return V(us-east-1) instead.
      - Affects only queries with O(bucket_facts=true) and O(bucket_facts.bucket_location=true).
    type: bool
    default: False
    version_added: 1.4.0
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Note: Only AWS S3 is currently supported

# Lists all S3 buckets
- amazon.aws.s3_bucket_info:
  register: result

# Retrieve detailed bucket information
- amazon.aws.s3_bucket_info:
    # Show only buckets with name matching
    name_filter: your.testing
    # Choose facts to retrieve
    bucket_facts:
      # bucket_accelerate_configuration: true
      bucket_acl: true
      bucket_cors: true
      bucket_encryption: true
      # bucket_lifecycle_configuration: true
      bucket_location: true
      # bucket_logging: true
      # bucket_notification_configuration: true
      # bucket_ownership_controls: true
      # bucket_policy: true
      # bucket_policy_status: true
      # bucket_replication: true
      # bucket_request_payment: true
      # bucket_tagging: true
      # bucket_website: true
      # public_access_block: true
    transform_location: true
    register: result

# Print out result
- name: List buckets
  ansible.builtin.debug:
    msg: "{{ result['buckets'] }}"
"""

RETURN = r"""
buckets:
  description: A list of S3 buckets.
  returned: always
  type: complex
  contains:
    bucket_accelerate_configuration:
      description: The accelerate configuration of the bucket.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_accelerate_configuration=true).
      type: dict
      sample: {
                'Status': 'Enabled',
                'RequestCharged': 'requester'
              }
    name:
      description: Bucket name.
      returned: always
      type: str
      sample: "a-testing-bucket-name"
    creation_date:
      description: Bucket creation date timestamp.
      returned: always
      type: str
      sample: "2021-01-21T12:44:10+00:00"
    public_access_block:
      description: Bucket public access block configuration.
      returned: when O(bucket_facts=true) and RV(buckets.public_access_block=true).
      type: dict
      sample: {
                "PublicAccessBlockConfiguration": {
                    "BlockPublicAcls": true,
                    "BlockPublicPolicy": true,
                    "IgnorePublicAcls": true,
                    "RestrictPublicBuckets": true
                }
              }
      contains:
        PublicAccessBlockConfiguration:
          description: PublicAccessBlockConfiguration data.
          returned: when PublicAccessBlockConfiguration is defined for the bucket
          type: complex
          contains:
            BlockPublicAcls:
              description: BlockPublicAcls setting value.
              type: bool
            BlockPublicPolicy:
              description: BlockPublicPolicy setting value.
              type: bool
            IgnorePublicAcls:
              description: IgnorePublicAcls setting value.
              type: bool
            RestrictPublicBuckets:
              description: RestrictPublicBuckets setting value.
              type: bool
    bucket_name_filter:
      description: String used to limit buckets. See I(name_filter).
      returned: when O(name_filter) is defined
      type: str
      sample: "filter-by-this-string"
    bucket_acl:
      description: Bucket ACL configuration.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_acl=true).
      type: complex
      contains:
        Grants:
          description: List of ACL grants.
          type: list
          elements: dict
          sample: [
                    {
                        "Grantee": {
                            "DisplayName": "abc-aws-root+721234567890",
                            "ID": "1234567890cbc0f1234567890f1234567890c123456789012123456789091234",
                            "Type": "CanonicalUser"
                        },
                        "Permission": "FULL_CONTROL"
                    }
                  ]
          contains:
            Grantee:
              description: The person being granted permissions.
              type: dict
              contains:
                DisplayName:
                  description: Screen name of the grantee.
                  type: str
                ID:
                  description: The canonical user ID of the grantee.
                  type: str
                Type:
                  description: Type of grantee.
                  type: str
            Permission:
              description: Specifies the permission given to the grantee.
              type: str
        Owner:
          description: Bucket owner information.
          type: dict
          sample: {
                    "DisplayName": "abc-aws-root+721234567890",
                    "ID": "1234567890cbc0f1234567890f1234567890c123456789012123456789091234"
                  }
          contains:
            DisplayName:
              description: Bucket owner user display name.
              type: str
            ID:
              description: Bucket owner user ID.
              type: str
    bucket_cors:
      description: Bucket CORS configuration.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_cors=true)
      type: complex
      contains:
        CORSRules:
          description: Bucket CORS configuration.
          returned: When CORS rules are defined for the bucket
          type: list
          sample: []
    bucket_encryption:
      description: Bucket encryption configuration.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_encryption=true).
      type: complex
      sample: {
                "ServerSideEncryptionConfiguration": {
                    "Rules": [
                        {
                          "ApplyServerSideEncryptionByDefault": {
                              "SSEAlgorithm": "AES256"
                          },
                          "BucketKeyEnabled": true
                        }
                    ]
                  }
                }
      contains:
        ServerSideEncryptionConfiguration:
          description: ServerSideEncryptionConfiguration configuration.
          returned: when encryption is enabled on the bucket.
          type: complex
          contains:
            Rules:
              description: List of applied encryptio rules.
              returned: when encryption is enabled on the bucket.
              type: list
              elements: dict
              contains:
                ApplyServerSideEncryptionByDefault:
                  description: Specifies the default server-side encryption to apply to new objects in the bucket.
                  type: dict
                  contains:
                    SSEAlgorithm:
                      description: Server-side encryption algorithm to use for the default encryption.
                      type: str
                    KMSMasterKeyID:
                      description: Amazon Web Services Key Management Service (KMS) customer Amazon Web Services KMS key ID to use for the default encryption.
                      type: str
                BucketKeyEnabled:
                  description:
                    - Specifies whether Amazon S3 should use an S3 Bucket Key with server-side encryption using KMS (SSE-KMS) for new objects in the bucket.
                  type: bool
    bucket_lifecycle_configuration:
      description: Bucket lifecycle configuration settings.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_lifecycle_configuration=true).
      type: list
      elements: dict
      contains:
        Rules:
          description: List of lifecycle management rules.
          returned: when lifecycle configuration is present.
          type: list
          sample: [
                    {
                      "Status": "Enabled",
                      "ID": "example-rule"
                    }
                  ]
    bucket_location:
      description: Bucket location.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_location=true).
      type: dict
      sample: {
                "LocationConstraint": "us-east-1"
              }
      contains:
        LocationConstraint:
          description: AWS region.
          returned: always
          type: str
    bucket_logging:
      description: Server access logging configuration.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_logging=true).
      type: complex
      contains:
        LoggingEnabled:
          description: Server access logging configuration.
          returned: when server access logging is defined for the bucket.
          type: complex
          contains:
            TargetBucket:
              description: Target bucket name.
              returned: always
              type: str
              sample: "logging-bucket-name"
            TargetPrefix:
              description: Prefix in target bucket.
              returned: always
              type: str
              sample: ""
    bucket_notification_configuration:
      description: Bucket notification settings.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_notification_configuration=true).
      type: complex
      contains:
        TopicConfigurations:
          description: List of notification events configurations.
          returned: when at least one notification is configured.
          type: list
          sample: [
                      {
                          'Id': 'topic-config-dev001',
                          'TopicArn': 'arn:aws:sns:us-west-1:721234567890:sns-dev001-topic',
                          'Events': [
                              "s3:ObjectCreated:Copy"
                          ],
                          'Filter': {
                              'Key': {
                                  'FilterRules': [
                                      {
                                            "Name": "Prefix",
                                            "Value": "test/"
                                      },
                                  ]
                              }
                          }
                      },
                    ]
    bucket_ownership_controls:
      description: Preffered object ownership settings.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_ownership_controls=true)
      type: complex
      sample: {
                "OwnershipControls": {
                  "Rules": [
                      {
                        "ObjectOwnership": "BucketOwnerEnforced"
                      }
                  ]
                }
              }
      contains:
        OwnershipControls:
          description: Object ownership settings.
          returned: when ownership controls are defined for the bucket.
          type: complex
          contains:
            Rules:
              description: List of ownership rules.
              returned: when ownership rule is defined.
              type: list
    bucket_policy:
      description: Bucket policy contents.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_policy=true).
      type: str
      sample: '{"Version":"2012-10-17","Statement":[{"Sid":"AddCannedAcl","Effect":"Allow",..}}]}'
    bucket_policy_status:
      description: Status of bucket policy.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_policy_status=true).
      type: dict
      contains:
        PolicyStatus:
          description: Status of bucket policy.
          returned: when bucket policy is present.
          type: dict
          contains:
            IsPublic:
              description: Report bucket policy public status.
              returned: when bucket policy is present.
              type: bool
              sample: True
    bucket_replication:
      description: Replication configuration settings.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_replication=true).
      type: complex
      contains:
        Role:
          description: IAM role used for replication.
          returned: when replication rule is defined.
          type: str
          sample: "arn:aws:iam::123:role/example-role"
        Rules:
          description: List of replication rules.
          returned: when replication rule is defined.
          type: list
          sample: [{ "ID": "rule-1", "Filter": "{}" }]
    bucket_request_payment:
      description: Requester pays setting.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_request_payment=true)
      type: complex
      sample: {
                "Payer": "BucketOwner"
              }
      contains:
        Payer:
          description: Current payer.
          returned: always
          type: str
    bucket_tagging:
      description: Bucket tags.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_tagging=true).
      type: dict
      sample: { "Tag1": "Value1", "Tag2": "Value2" }
    bucket_website:
      description: Static website hosting.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_website=true).
      type: complex
      contains:
        ErrorDocument:
          description: Object serving as HTTP error page.
          returned: when static website hosting is enabled.
          type: dict
          sample: { "Key": "error.html" }
        IndexDocument:
          description: Object serving as HTTP index page.
          returned: when static website hosting is enabled.
          type: dict
          sample: { "Suffix": "error.html" }
        RedirectAllRequestsTo:
          description: Website redict settings.
          returned: when redirect requests is configured.
          type: complex
          contains:
            HostName:
              description: Hostname to redirect.
              returned: always
              type: str
              sample: "www.example.com"
            Protocol:
              description: Protocol used for redirect.
              returned: always
              type: str
              sample: "https"
    bucket_versioning:
      description:
        - The versioning state of the bucket.
        - This will also specify whether MFA delete is enabled in the bucket versioning configuration.
          if only the bucket has been configured with MFA delete.
      returned: when O(bucket_facts=true) and RV(buckets.bucket_versioning=true).
      type: dict
      sample: { "Status": "Enabled" }
      version_added: 7.2.0
"""

import typing

if typing.TYPE_CHECKING:
    from typing import Dict
    from typing import List

    from ansible_collections.amazon.aws.plugins.module_utils.botocore import ClientType

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.s3 import AnsibleS3Error
from ansible_collections.amazon.aws.plugins.module_utils.s3 import S3ErrorHandler
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_bucket_location
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_bucket_accelerate_configuration
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_bucket_acl
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_bucket_encryption
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_bucket_ownership_controls
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_bucket_policy
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_bucket_public_access_block
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_bucket_request_payment
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_bucket_tagging
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_bucket_versioning
from ansible_collections.amazon.aws.plugins.module_utils.s3 import list_s3_buckets


def get_bucket_list(
    module: AnsibleAWSModule,
    connection: ClientType,
    name: str = "",
    name_filter: str = "",
) -> List[Dict]:
    """
    Return result of list_buckets json encoded
    Filter only buckets matching 'name' or name_filter if defined
    :param module:
    :param connection:
    :return:
    """
    buckets = []
    filtered_buckets = []
    final_buckets = []

    # Get all buckets
    # camel_dict_to_snake_dict doesn't support being passed a list,
    # so we wrap the bucket list in a dictionary temporarily
    buckets = camel_dict_to_snake_dict({"Buckets": list_s3_buckets(connection)})["buckets"]

    # Filter buckets if requested
    if name_filter:
        for bucket in buckets:
            if name_filter in bucket["name"]:
                filtered_buckets.append(bucket)
    elif name:
        for bucket in buckets:
            if name == bucket["name"]:
                filtered_buckets.append(bucket)

    # Return proper list (filtered or all)
    if name or name_filter:
        final_buckets = filtered_buckets
    else:
        final_buckets = buckets
    return final_buckets


def get_buckets_facts(
    connection: ClientType, buckets: List[Dict], requested_facts: Dict, transform_location: bool
) -> List[Dict]:
    """
    Retrieve additional information about S3 buckets
    """
    full_bucket_list = []
    # Iterate over all buckets and append Retrieved facts to bucket
    for bucket in buckets:
        bucket.update(get_bucket_details(connection, bucket["name"], requested_facts, transform_location))
        full_bucket_list.append(bucket)

    return full_bucket_list


def get_bucket_details(connection: ClientType, name: str, requested_facts: Dict, transform_location: bool) -> Dict:
    """
    Execute all enabled S3API get calls for selected bucket using module_utils functions.
    """
    # Map bucket_facts keys to module_utils functions
    bucket_function_map = {
        "bucket_accelerate_configuration": get_s3_bucket_accelerate_configuration,
        "bucket_acl": get_s3_bucket_acl,
        "bucket_encryption": get_s3_bucket_encryption,
        "bucket_ownership_controls": get_s3_bucket_ownership_controls,
        "bucket_policy": get_s3_bucket_policy,
        "public_access_block": get_s3_bucket_public_access_block,
        "bucket_request_payment": get_s3_bucket_request_payment,
        "bucket_tagging": get_s3_bucket_tagging,
        "bucket_versioning": get_s3_bucket_versioning,
    }

    all_facts = {}

    for key in requested_facts:
        if not requested_facts[key]:
            continue

        all_facts[key] = {}
        try:
            if key == "bucket_location":
                # Special handling for bucket_location with transform_location parameter
                location_data = get_bucket_location(connection, name)
                # Transform 'None' LocationConstraint to 'us-east-1' if requested
                if transform_location and location_data.get("LocationConstraint") is None:
                    location_data["LocationConstraint"] = "us-east-1"
                all_facts[key] = location_data
            elif key in bucket_function_map:
                # Use module_utils function
                all_facts[key] = bucket_function_map[key](connection, name)
            else:
                # For properties not in module_utils, use dynamic call
                all_facts[key] = get_bucket_property(name, connection, key)
        except AnsibleS3Error:
            # Silent failure for missing/inaccessible bucket properties (backward compatibility)
            pass

    return all_facts


@S3ErrorHandler.list_error_handler("get bucket property", {})
@AWSRetry.jittered_backoff(max_delay=120, catch_extra_error_codes=["NoSuchBucket", "OperationAborted"])
def get_bucket_property(name: str, connection: ClientType, get_api_name: str) -> Dict:
    """
    Get bucket property
    """
    api_call = "get_" + get_api_name
    api_function = getattr(connection, api_call)
    data = api_function(Bucket=name)

    # Strip response metadata (not needed)
    data.pop("ResponseMetadata", None)
    return data


def main() -> None:
    """
    Get list of S3 buckets
    :return:
    """
    argument_spec = dict(
        name=dict(type="str", default=""),
        name_filter=dict(type="str", default=""),
        bucket_facts=dict(
            type="dict",
            options=dict(
                bucket_accelerate_configuration=dict(type="bool", default=False),
                bucket_acl=dict(type="bool", default=False),
                bucket_cors=dict(type="bool", default=False),
                bucket_encryption=dict(type="bool", default=False),
                bucket_lifecycle_configuration=dict(type="bool", default=False),
                bucket_location=dict(type="bool", default=False),
                bucket_logging=dict(type="bool", default=False),
                bucket_notification_configuration=dict(type="bool", default=False),
                bucket_ownership_controls=dict(type="bool", default=False),
                bucket_policy=dict(type="bool", default=False),
                bucket_policy_status=dict(type="bool", default=False),
                bucket_replication=dict(type="bool", default=False),
                bucket_request_payment=dict(type="bool", default=False),
                bucket_tagging=dict(type="bool", default=False),
                bucket_website=dict(type="bool", default=False),
                public_access_block=dict(type="bool", default=False),
                bucket_versioning=dict(type="bool", default=False),
            ),
        ),
        transform_location=dict(type="bool", default=False),
    )

    # Ensure we have an empty dict
    result = {}

    # Define mutually exclusive options
    mutually_exclusive = [
        ["name", "name_filter"],
    ]

    # Including ec2 argument spec
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    # Get parameters
    name = module.params.get("name")
    name_filter = module.params.get("name_filter")
    requested_facts = module.params.get("bucket_facts")
    transform_location = module.params.get("transform_location")

    # Set up connection
    connection = module.client("s3")

    # Get basic bucket list (name + creation date)
    bucket_list = get_bucket_list(module, connection, name, name_filter)

    # Add information about name/name_filter to result
    if name:
        result["bucket_name"] = name
    elif name_filter:
        result["bucket_name_filter"] = name_filter

    # Gather detailed information about buckets if requested
    bucket_facts = module.params.get("bucket_facts")
    if bucket_facts:
        result["buckets"] = get_buckets_facts(connection, bucket_list, requested_facts, transform_location)
    else:
        result["buckets"] = bucket_list

    module.exit_json(msg="Retrieved s3 info.", **result)


# MAIN
if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: mq_broker
version_added: 6.0.0
short_description: MQ broker management
description:
  - Create/update/delete a broker.
  - Reboot a broker.
author:
  - FCO (@fotto)
options:
  broker_name:
    description:
      - The Name of the MQ broker to work on.
    type: str
    required: true
  state:
    description:
      - "C(present): Create/update broker."
      - "C(absent): Delete broker."
      - "C(restarted): Reboot broker."
    choices: [ 'present', 'absent', 'restarted' ]
    default: present
    type: str
  deployment_mode:
    description:
      - Set broker deployment type.
      - Can be used only during creation.
      - Defaults to C(SINGLE_INSTANCE).
    choices: [ 'SINGLE_INSTANCE', 'ACTIVE_STANDBY_MULTI_AZ', 'CLUSTER_MULTI_AZ' ]
    type: str
  use_aws_owned_key:
    description:
      - Must be set to C(false) if I(kms_key_id) is provided as well.
      - Can be used only during creation.
      - Defaults to C(true).
    type: bool
  kms_key_id:
    description:
      - Use referenced key to encrypt broker data at rest.
      - Can be used only during creation.
    type: str
  engine_type:
    description:
      - Set broker engine type.
      - Can be used only during creation.
      - Defaults to C(ACTIVEMQ).
    choices: [ 'ACTIVEMQ', 'RABBITMQ' ]
    type: str
  maintenance_window_start_time:
    description:
      - Set maintenance window for automatic minor upgrades.
      - Can be used only during creation.
      - Not providing any value means "no maintenance window".
    type: dict
  publicly_accessible:
    description:
      - Allow/disallow public access.
      - Can be used only during creation.
      - Defaults to C(false).
    type: bool
  storage_type:
    description:
      - Set underlying storage type.
      - Can be used only during creation.
      - Defaults to C(EFS).
    choices: [ 'EBS', 'EFS' ]
    type: str
  subnet_ids:
    description:
      - Defines where deploy broker instances to.
      - Minimum required number depends on deployment type.
      - Can be used only during creation.
    type: list
    elements: str
  users:
    description:
      - This parameter allows to use a custom set of initial user(s).
      - M(community.aws.mq_user) is the preferred way to manage (local) users
        however a broker cannot be created without any user.
      - If nothing is specified a default C(admin) user will be created along with brokers.
      - Can be used only during creation.  Use M(community.aws.mq_user) module for updates.
    type: list
    elements: dict
  tags:
    description:
      - Tag newly created brokers.
      - Can be used only during creation.
    type: dict
  authentication_strategy:
    description: Choose between locally and remotely managed users.
    choices: [ 'SIMPLE', 'LDAP' ]
    type: str
  auto_minor_version_upgrade:
    description: Allow/disallow automatic minor version upgrades.
    type: bool
    default: true
  engine_version:
    description:
      - Set engine version of broker.
      - The special value C(latest) will pick the latest available version.
      - The special value C(latest) is ignored on update.
    type: str
  host_instance_type:
    description: Instance type of broker instances.
    type: str
  enable_audit_log:
    description: Enable/disable to push audit logs to AWS CloudWatch.
    type: bool
    default: false
  enable_general_log:
    description: Enable/disable to push general logs to AWS CloudWatch.
    type: bool
    default: false
  security_groups:
    description:
      - Associate security groups with broker.
      - At least one must be provided during creation.
    type: list
    elements: str
  wait:
    description:
      - Specifies whether the module waits for the desired C(state).
      - The time to wait can be controlled by setting I(wait_timeout).
    type: bool
    default: false
    version_added: 7.1.0
  wait_timeout:
    description:
      - How long to wait (in seconds) for the broker to reach the desired state if I(wait=true).
    default: 900
    type: int
    version_added: 7.1.0

extends_documentation_fragment:
  - amazon.aws.boto3
  - amazon.aws.common.modules
  - amazon.aws.region.modules
"""


EXAMPLES = r"""
- name: create broker (if missing) with minimal required parameters
  community.aws.mq_broker:
    broker_name: "{{ broker_name }}"
    security_groups:
      - sg_xxxxxxx
    subnet_ids:
      - subnet_xxx
      - subnet_yyy
    register: result

- set_fact:
    broker_id: "{{ result.broker['BrokerId'] }}"

- name: use mq_broker_info to wait until broker is ready
  community.aws.mq_broker_info:
    broker_id: "{{ broker_id }}"
  register: result
  until: "result.broker['BrokerState'] == 'RUNNING'"
  retries: 15
  delay: 60

- name: create or update broker with almost all parameter set including credentials
  community.aws.mq_broker:
    broker_name: "my_broker_2"
    state: present
    deployment_mode: 'ACTIVE_STANDBY_MULTI_AZ'
    use_aws_owned_key: false
    kms_key_id: 'my-precreted-key-id'
    engine_type: 'ACTIVEMQ'
    maintenance_window_start_time:
      DayOfWeek: 'MONDAY'
      TimeOfDay: '03:15'
      TimeZone: 'Europe/Berlin'
    publicly_accessible: true
    storage_type: 'EFS'
    security_groups:
      - sg_xxxxxxx
    subnet_ids:
      - subnet_xxx
      - subnet_yyy
    users:
      - Username: 'initial-user'
        Password': 'plain-text-password'
        ConsoleAccess: true
    tags:
      env: Test
      creator: ansible
    authentication_strategy: 'SIMPLE'
    auto_minor_version_upgrade: true
    engine_version: "5.15.13"
    host_instance_type: 'mq.t3.micro'
    enable_audit_log: true
    enable_general_log: true

- name: reboot a broker
  community.aws.mq_broker:
    broker_name: "my_broker_2"
    state: restarted

- name: delete a broker
  community.aws.mq_broker:
    broker_name: "my_broker_2"
    state: absent
"""

RETURN = r"""
broker:
  description:
    - "All API responses are converted to snake yaml except 'Tags'"
    - "'state=present': API response of create_broker() or update_broker() call"
    - "'state=absent': result of describe_broker() call before delete_broker() is triggerd"
    - "'state=restarted': result of describe_broker() after reboot has been triggered"
  type: dict
  returned: success
"""

try:
    import botocore
except ImportError:
    # handled by AnsibleAWSModule
    pass

from time import sleep
from time import time

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule

PARAMS_MAP = {
    "authentication_strategy": "AuthenticationStrategy",
    "auto_minor_version_upgrade": "AutoMinorVersionUpgrade",
    "broker_name": "BrokerName",
    "deployment_mode": "DeploymentMode",
    "use_aws_owned_key": "EncryptionOptions/UseAwsOwnedKey",
    "kms_key_id": "EncryptionOptions/KmsKeyId",
    "engine_type": "EngineType",
    "engine_version": "EngineVersion",
    "host_instance_type": "HostInstanceType",
    "enable_audit_log": "Logs/Audit",
    "enable_general_log": "Logs/General",
    "maintenance_window_start_time": "MaintenanceWindowStartTime",
    "publicly_accessible": "PubliclyAccessible",
    "security_groups": "SecurityGroups",
    "storage_type": "StorageType",
    "subnet_ids": "SubnetIds",
    "users": "Users",
    "tags": "Tags",
}


DEFAULTS = {
    "authentication_strategy": "SIMPLE",
    "auto_minor_version_upgrade": False,
    "deployment_mode": "SINGLE_INSTANCE",
    "use_aws_owned_key": True,
    "engine_type": "ACTIVEMQ",
    "engine_version": "latest",
    "host_instance_type": "mq.t3.micro",
    "enable_audit_log": False,
    "enable_general_log": False,
    "publicly_accessible": False,
    "storage_type": "EFS",
}

CREATE_ONLY_PARAMS = [
    "deployment_mode",
    "use_aws_owned_key",
    "kms_key_id",
    "engine_type",
    "maintenance_window_start_time",
    "publicly_accessible",
    "storage_type",
    "subnet_ids",
    "users",
    "tags",
]


def _set_kwarg(kwargs, key, value):
    mapped_key = PARAMS_MAP[key]
    if "/" in mapped_key:
        key_list = mapped_key.split("/")
        key_list.reverse()
    else:
        key_list = [mapped_key]
    data = kwargs
    while len(key_list) > 1:
        this_key = key_list.pop()
        if this_key not in data:
            data[this_key] = {}
        #
        data = data[this_key]
    data[key_list[0]] = value


def _fill_kwargs(module, apply_defaults=True, ignore_create_params=False):
    kwargs = {}
    if apply_defaults:
        for p_name, p_value in DEFAULTS.items():
            _set_kwarg(kwargs, p_name, p_value)
    for p_name in module.params:
        if ignore_create_params and p_name in CREATE_ONLY_PARAMS:
            # silently ignore CREATE_ONLY_PARAMS on update to
            # make playbooks idempotent
            continue
        if p_name in PARAMS_MAP and module.params[p_name] is not None:
            _set_kwarg(kwargs, p_name, module.params[p_name])
        else:
            # ignore
            pass
    return kwargs


def __list_needs_change(current, desired):
    if len(current) != len(desired):
        return True
    for index, value in enumerate(current):
        if value != desired[index]:
            return True
    #
    return False


def __dict_needs_change(current, desired):
    # values contained in 'current' but not specified in 'desired' are ignored
    # value contained in 'desired' but not in 'current' (unsupported attributes) are ignored
    for key in desired:
        if key in current:
            if desired[key] != current[key]:
                return True
    #
    return False


def _needs_change(current, desired):
    needs_change = False
    for key in desired:
        current_value = current[key]
        desired_value = desired[key]
        if isinstance(current_value, (int, str, bool)):
            if current_value != desired_value:
                needs_change = True
                break
        elif isinstance(current_value, list):
            # assumption: all 'list' type settings we allow changes for have scalar values
            if __list_needs_change(current_value, desired_value):
                needs_change = True
                break
        elif isinstance(current_value, dict):
            # assumption: all 'dict' type settings we allow changes for have scalar values
            if __dict_needs_change(current_value, desired_value):
                needs_change = True
                break
        else:
            # unexpected type
            needs_change = True
            break
    #
    return needs_change


def get_latest_engine_version(conn, module, engine_type):
    try:
        response = conn.describe_broker_engine_types(EngineType=engine_type)
        return response["BrokerEngineTypes"][0]["EngineVersions"][0]["Name"]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't list engine versions")


def get_broker_id(conn, module):
    try:
        broker_name = module.params["broker_name"]
        broker_id = None
        response = conn.list_brokers(MaxResults=100)
        for broker in response["BrokerSummaries"]:
            if broker["BrokerName"] == broker_name:
                broker_id = broker["BrokerId"]
                break
        return broker_id
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't list broker brokers.")


def get_broker_info(conn, module, broker_id):
    try:
        return conn.describe_broker(BrokerId=broker_id)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't get broker details.")


def wait_for_status(conn, module):
    interval_secs = 5
    timeout = module.params.get("wait_timeout", 900)
    broker_name = module.params.get("broker_name")
    desired_state = module.params.get("state")
    done = False

    paginator = conn.get_paginator("list_brokers")
    page_iterator = paginator.paginate(PaginationConfig={"MaxItems": 100, "PageSize": 100, "StartingToken": ""})
    wait_timeout = time() + timeout

    while wait_timeout > time():
        try:
            filtered_iterator = page_iterator.search(f"BrokerSummaries[?BrokerName == `{broker_name}`][]")
            broker_list = list(filtered_iterator)

            if module.check_mode:
                return

            if len(broker_list) < 1 and desired_state == "absent":
                done = True
                break

            if desired_state in ["present", "rebooted"] and broker_list[0]["BrokerState"] == "RUNNING":
                done = True
                break

            if broker_list[0]["BrokerState"] == "CREATION_FAILED":
                break

            sleep(interval_secs)

        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't paginate brokers.")

    if not done:
        module.fail_json(msg="desired state not reached")


def reboot_broker(conn, module, broker_id):
    wait = module.params.get("wait")

    try:
        response = conn.reboot_broker(BrokerId=broker_id)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't reboot broker.")

    if wait:
        wait_for_status(conn, module)

    return response


def delete_broker(conn, module, broker_id):
    wait = module.params.get("wait")

    try:
        response = conn.delete_broker(BrokerId=broker_id)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't delete broker.")

    if wait:
        wait_for_status(conn, module)

    return response


def create_broker(conn, module):
    kwargs = _fill_kwargs(module)
    wait = module.params.get("wait")

    if "EngineVersion" in kwargs and kwargs["EngineVersion"] == "latest":
        kwargs["EngineVersion"] = get_latest_engine_version(conn, module, kwargs["EngineType"])
    if kwargs["AuthenticationStrategy"] == "LDAP":
        module.fail_json(msg="'AuthenticationStrategy=LDAP' not supported, yet")
    if "Users" not in kwargs:
        # add some stupid default (cannot create broker without any users)
        kwargs["Users"] = [{"Username": "admin", "Password": "adminPassword", "ConsoleAccess": True, "Groups": []}]
    if "EncryptionOptions" in kwargs and "UseAwsOwnedKey" in kwargs["EncryptionOptions"]:
        kwargs["EncryptionOptions"]["UseAwsOwnedKey"] = False
    #
    if "SecurityGroups" not in kwargs or len(kwargs["SecurityGroups"]) == 0:
        module.fail_json(msg="At least one security group must be specified on broker creation")
    #
    changed = True
    result = conn.create_broker(**kwargs)
    #
    if wait:
        wait_for_status(conn, module)

    return {"broker": camel_dict_to_snake_dict(result, ignore_list=["Tags"]), "changed": changed}


def update_broker(conn, module, broker_id):
    kwargs = _fill_kwargs(module, apply_defaults=False, ignore_create_params=True)
    wait = module.params.get("wait")
    # replace name with id
    broker_name = kwargs["BrokerName"]
    del kwargs["BrokerName"]
    kwargs["BrokerId"] = broker_id
    # get current state for comparison:
    api_result = get_broker_info(conn, module, broker_id)
    if api_result["BrokerState"] != "RUNNING":
        module.fail_json(
            msg=f"Cannot trigger update while broker ({broker_id}) is in state {api_result['BrokerState']}",
        )
    # engine version of 'latest' is taken as "keep current one"
    # i.e. do not request upgrade on playbook rerun
    if "EngineVersion" in kwargs and kwargs["EngineVersion"] == "latest":
        kwargs["EngineVersion"] = api_result["EngineVersion"]
    result = {"broker_id": broker_id, "broker_name": broker_name}
    changed = False
    if _needs_change(api_result, kwargs):
        changed = True
        if not module.check_mode:
            api_result = conn.update_broker(**kwargs)
        #
    #
    if wait:
        wait_for_status(conn, module)

    return {"broker": result, "changed": changed}


def ensure_absent(conn, module):
    result = {"broker_name": module.params["broker_name"], "broker_id": None}
    if module.check_mode:
        return {"broker": camel_dict_to_snake_dict(result, ignore_list=["Tags"]), "changed": True}
    broker_id = get_broker_id(conn, module)
    result["broker_id"] = broker_id

    if not broker_id:
        # silently ignore delete of unknown broker (to make it idempotent)
        return {"broker": result, "changed": False}

    try:
        # check for pending delete (small race condition possible here
        api_result = get_broker_info(conn, module, broker_id)
        if api_result["BrokerState"] == "DELETION_IN_PROGRESS":
            return {"broker": result, "changed": False}
        delete_broker(conn, module, broker_id)
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)

    return {"broker": result, "changed": True}


def ensure_present(conn, module):
    if module.check_mode:
        return {"broker": {"broker_arn": "fakeArn", "broker_id": "fakeId"}, "changed": True}

    broker_id = get_broker_id(conn, module)
    if broker_id:
        return update_broker(conn, module, broker_id)

    return create_broker(conn, module)


def main():
    argument_spec = dict(
        broker_name=dict(required=True, type="str"),
        state=dict(default="present", choices=["present", "absent", "restarted"]),
        wait=dict(default=False, type="bool"),
        wait_timeout=dict(default=900, type="int"),
        # parameters only allowed on create
        deployment_mode=dict(choices=["SINGLE_INSTANCE", "ACTIVE_STANDBY_MULTI_AZ", "CLUSTER_MULTI_AZ"]),
        use_aws_owned_key=dict(type="bool"),
        kms_key_id=dict(type="str"),
        engine_type=dict(choices=["ACTIVEMQ", "RABBITMQ"], type="str"),
        maintenance_window_start_time=dict(type="dict"),
        publicly_accessible=dict(type="bool"),
        storage_type=dict(choices=["EBS", "EFS"]),
        subnet_ids=dict(type="list", elements="str"),
        users=dict(type="list", elements="dict"),
        tags=dict(type="dict"),
        # parameters allowed on update as well
        authentication_strategy=dict(choices=["SIMPLE", "LDAP"]),
        auto_minor_version_upgrade=dict(default=True, type="bool"),
        engine_version=dict(type="str"),
        host_instance_type=dict(type="str"),
        enable_audit_log=dict(default=False, type="bool"),
        enable_general_log=dict(default=False, type="bool"),
        security_groups=dict(type="list", elements="str"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    connection = module.client("mq")

    if module.params["state"] == "present":
        try:
            compound_result = ensure_present(connection, module)
        except botocore.exceptions.ClientError as e:
            module.fail_json_aws(e)
        #
        module.exit_json(**compound_result)

    if module.params["state"] == "absent":
        try:
            compound_result = ensure_absent(connection, module)
        except botocore.exceptions.ClientError as e:
            module.fail_json_aws(e)
        #
        module.exit_json(**compound_result)

    if module.params["state"] == "restarted":
        broker_id = get_broker_id(connection, module)
        if module.check_mode:
            module.exit_json(broker={"broker_id": broker_id if broker_id else "fakeId"}, changed=True)
        if not broker_id:
            module.fail_json(
                msg="Cannot find broker with name {module.params['broker_name']}.",
            )
        try:
            changed = True
            if not module.check_mode:
                reboot_broker(connection, module, broker_id)
            #
            result = get_broker_info(connection, module, broker_id)
        except botocore.exceptions.ClientError as e:
            module.fail_json_aws(e)
        module.exit_json(broker=result, changed=changed)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_dhcp_option
short_description: Manage DHCP Option (dhcp:Option)
description:
- Manage DHCP Options for DHCP Option Policies on Cisco ACI fabrics.
- The DHCP option is used to supply DHCP clients with configuration parameters such as a domain, name server, subnet, and network address.
  DHCP provides a framework for passing configuration information to clients on a TCP/IP network.
  The configuration parameters, and other control information, are carried in tagged data items that are stored in the options field of a DHCP message.
  The data items themselves are also called options. You can view, set, unset, and edit DHCP option values.
  When you set an option value, the DHCP server replaces any existing value or creates a new one as needed for the given option name.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  dhcp_option_policy:
    description:
    - The name of an existing DHCP Option Policy.
    type: str
    aliases: [ dhcp_option_policy_name ]
  dhcp_option:
    description:
    - The name of the DHCP Option.
    type: str
    aliases: [ dhcp_option_name, name ]
  data:
    description:
    - The value of the DHCP Option.
    type: str
  id:
    description:
    - The DHCP Option ID.
    type: int
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Add a new DHCP Option
  cisco.aci.aci_dhcp_option:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    dhcp_option_policy: my_dhcp_option_policy
    dhcp_option: my_dhcp_option
    id: 1
    data: 82
    state: present
  delegate_to: localhost

- name: Delete an DHCP Option
  cisco.aci.aci_dhcp_option:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    dhcp_option_policy: my_dhcp_option_policy
    dhcp_option: my_dhcp_option
    state: absent
  delegate_to: localhost

- name: Query a DHCP Option
  cisco.aci.aci_dhcp_option:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    dhcp_option_policy: my_dhcp_option_policy
    dhcp_option: my_dhcp_option
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all DHCP Options in my_dhcp_option_policy
  cisco.aci.aci_dhcp_option:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    dhcp_option_policy: my_dhcp_option_policy
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        dhcp_option_policy=dict(type="str", aliases=["dhcp_option_policy_name"]),
        dhcp_option=dict(type="str", aliases=["dhcp_option_name", "name"]),
        data=dict(type="str"),
        id=dict(type="int"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "dhcp_option_policy", "dhcp_option"]],
            ["state", "present", ["tenant", "dhcp_option_policy", "dhcp_option"]],
        ],
    )

    tenant = module.params.get("tenant")
    dhcp_option_policy = module.params.get("dhcp_option_policy")
    dhcp_option = module.params.get("dhcp_option")
    data = module.params.get("data")
    id = module.params.get("id")
    state = module.params.get("state")

    aci = ACIModule(module)

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="dhcpOptionPol",
            aci_rn="dhcpoptpol-{0}".format(dhcp_option_policy),
            module_object=dhcp_option_policy,
            target_filter={"name": dhcp_option_policy},
        ),
        subclass_2=dict(
            aci_class="dhcpOption",
            aci_rn="opt-{0}".format(dhcp_option),
            module_object=dhcp_option,
            target_filter={"name": dhcp_option},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="dhcpOption",
            class_config=dict(
                name=dhcp_option,
                data=data,
                id=id,
            ),
        )

        aci.get_diff(aci_class="dhcpOption")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_lan_connectivity_policy_info
short_description: Gather information about LAN Connectivity Policies in Cisco Intersight
description:
  - Gather information about LAN Connectivity Policies in L(Cisco Intersight,https://intersight.com).
  - Information can be filtered by O(organization) and O(name).
  - If no filters are passed, all LAN Connectivity Policies will be returned.
  - Each policy includes its associated vNICs as part of the response.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization the LAN Connectivity Policy belongs to.
    type: str
  name:
    description:
      - The name of the LAN Connectivity Policy to gather information from.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Fetch a specific LAN Connectivity Policy by name
  cisco.intersight.intersight_lan_connectivity_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "datacenter-lan-policy"

- name: Fetch all LAN Connectivity Policies in a specific Organization
  cisco.intersight.intersight_lan_connectivity_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"

- name: Fetch all LAN Connectivity Policies
  cisco.intersight.intersight_lan_connectivity_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": [
    {
        "Name": "datacenter-lan-policy",
        "ObjectType": "vnic.LanConnectivityPolicy",
        "TargetPlatform": "Standalone",
        "AzureQosEnabled": false,
        "IqnAllocationType": "None",
        "PlacementMode": "custom",
        "StaticIqnName": "",
        "vNICs": [
            {
                "Name": "eth0",
                "ObjectType": "vnic.EthIf",
                "Order": 0,
                "Placement": {
                    "Id": "1",
                    "Uplink": 0,
                    "PciLink": 0
                },
                "Cdn": {
                    "Source": "vnic"
                },
                "EthNetworkPolicy": "6898934a010c5abf01f88125",
                "EthQosPolicy": "6898934a010c5abf01f88126",
                "EthAdapterPolicy": "68989358010c5abf01f89569"
            },
            {
                "Name": "eth1-usnic",
                "ObjectType": "vnic.EthIf",
                "Order": 1,
                "Placement": {
                    "Id": "2",
                    "Uplink": 0,
                    "PciLink": 0
                },
                "Cdn": {
                    "Source": "user",
                    "Value": "custom-eth1"
                },
                "UsnicSettings": {
                    "Count": 15,
                    "Cos": 3,
                    "UsnicAdapterPolicy": "68989358010c5abf01f89569"
                },
                "EthNetworkPolicy": "6898934a010c5abf01f88125",
                "EthQosPolicy": "6898934a010c5abf01f88126",
                "EthAdapterPolicy": "68989358010c5abf01f89569"
            }
        ],
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    },
    {
        "Name": "fi-attached-policy",
        "ObjectType": "vnic.LanConnectivityPolicy",
        "TargetPlatform": "FIAttached",
        "AzureQosEnabled": true,
        "IqnAllocationType": "Pool",
        "PlacementMode": "auto",
        "IqnPool": "6898a32d6f62693301a90d5c",
        "vNICs": [
            {
                "Name": "vnic-fi-attached",
                "ObjectType": "vnic.EthIf",
                "MacAddressType": "POOL",
                "MacPool": "6898a2fc6962753101f5bb41",
                "Order": 0,
                "Placement": {
                    "SwitchId": "A",
                    "AutoSlotId": true,
                    "AutoPciLink": true
                },
                "Cdn": {
                    "Source": "vnic"
                },
                "FailoverEnabled": false,
                "FabricEthNetworkGroupPolicy": [
                    "6898a32d6f62693301a90d5c"
                ],
                "FabricEthNetworkControlPolicy": "6898a3426f62693301a92527",
                "EthQosPolicy": "6898934a010c5abf01f88126",
                "EthAdapterPolicy": "68989358010c5abf01f89569"
            }
        ],
        "Tags": [
            {
                "Key": "Site",
                "Value": "Datacenter1"
            }
        ]
    }
  ]
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/vnic/LanConnectivityPolicies'

    query_params = intersight.set_query_params()

    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )

    # Fetch vNICs for each LAN connectivity policy
    lan_policies = intersight.result['api_response']
    if isinstance(lan_policies, list):
        for policy in lan_policies:
            if policy.get('Moid'):
                # Fetch vNICs for this policy using LanConnectivityPolicy.Moid filter
                vnics_query_params = {
                    '$filter': f"LanConnectivityPolicy.Moid eq '{policy['Moid']}'"
                }

                # Create a temporary intersight instance for vNICs query
                temp_intersight = IntersightModule(module)
                temp_intersight.get_resource(
                    resource_path='/vnic/EthIfs',
                    query_params=vnics_query_params,
                    return_list=True
                )

                # Add vNICs to the policy
                policy['vNICs'] = temp_intersight.result.get('api_response', [])
    elif isinstance(lan_policies, dict) and lan_policies.get('Moid'):
        # Single policy case
        vnics_query_params = {
            '$filter': f"LanConnectivityPolicy.Moid eq '{lan_policies['Moid']}'"
        }

        # Create a temporary intersight instance for vNICs query
        temp_intersight = IntersightModule(module)
        temp_intersight.get_resource(
            resource_path='/vnic/EthIfs',
            query_params=vnics_query_params,
            return_list=True
        )

        # Add vNICs to the policy
        lan_policies['vNICs'] = temp_intersight.result.get('api_response', [])

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()
